@@ -59,7 +59,8 @@ import sys
 
 # relax module imports.
 from pipe_control.grace import write_xy_data, write_xy_header
-from lib.physical_constants import dipolar_constant, g1H, g13C
+from lib.periodic_table import periodic_table
+from lib.physical_constants import dipolar_constant
 from prompt.interpreter import Interpreter
 from lib.errors import RelaxError
 from lib.io import mkdir_nofail
@@ -573,7 +574,7 @@ class Stereochem_analysis:
                 self.interpreter.structure.get_pos(ave_pos=False)
 
                 # Calculate the average NOE potential.
-                self.interpreter.calc()
+                self.interpreter.minimise.calculate()
 
                 # Sum the violations.
                 cdp.sum_viol = 0.0
@@ -606,7 +607,7 @@ class Stereochem_analysis:
         # The dipolar constant.
         d = 0.0
         if self.bond_length != None:
-            d = 3.0 / (2.0*pi) * dipolar_constant(g13C, g1H, self.bond_length)
+            d = 3.0 / (2.0*pi) * dipolar_constant(periodic_table.gyromagnetic_ratio('13C'), periodic_table.gyromagnetic_ratio('1H'), self.bond_length)
 
         # Create a directory for the save files.
         dir = self.results_dir + sep + "RDC_%s_results" % self.rdc_name
@@ -681,8 +682,8 @@ class Stereochem_analysis:
                 self.interpreter.interatom.unit_vectors(ave=False)
 
                 # Minimisation.
-                #grid_search(inc=4)
-                self.interpreter.minimise("simplex", constraints=False)
+                #minimise.grid_search(inc=4)
+                self.interpreter.minimise.execute("simplex", constraints=False)
 
                 # Store and write out the Q factors.
                 q_factors.append([cdp.q_rdc, ens])
