@@ -202,6 +202,36 @@ class Spectra_list(Base_list):
             uf_store['relax_disp.spin_lock_field'](spectrum_id=id)
 
 
+    def action_relax_disp_spin_lock_offset(self, event=None, item=None):
+        """Launch the relax_disp.spin_lock_offset user function.
+
+        @keyword event: The wx event.
+        @type event:    wx event
+        @keyword item:  This is for debugging purposes only, to allow the GUI tests to select items without worrying about OS dependent wxPython bugs.
+        @type item:     None or int
+        """
+
+        # The current selection.
+        if item == None:
+            item = self.element.GetFirstSelected()
+
+        # The spectrum ID.
+        id = gui_to_str(self.element.GetItemText(item))
+
+        # The offset.
+        offset = None
+        offset_flag = False
+        if hasattr(cdp, 'spin_lock_offset') and id in cdp.spin_lock_offset.keys():
+            offset = cdp.spin_lock_offset[id]
+            offset_flag = True
+
+        # Launch the dialog.
+        if offset_flag:
+            uf_store['relax_disp.spin_lock_offset'](offset=offset, spectrum_id=id)
+        else:
+            uf_store['relax_disp.spin_lock_offset'](spectrum_id=id)
+
+
     def action_relax_fit_relax_time(self, event=None, item=None):
         """Launch the relax_fit.relax_time user function.
 
@@ -298,6 +328,41 @@ class Spectra_list(Base_list):
         uf_store['spectrum.delete'](spectrum_id=id)
 
 
+    def action_spectrum_error_analysis(self, event):
+        """Launch the spectrum.error_analysis user function.
+
+        @param event:   The wx event.
+        @type event:    wx event
+        """
+
+        # The first item selected.
+        item = self.element.GetFirstSelected()
+
+        # Loop over the additional selections.
+        ids = []
+        while True:
+            # No selection.
+            if item == -1:
+                break
+
+            # Add the ID string to the list.
+            ids.append(gui_to_str(self.element.GetItemText(item)))
+
+            # Get the next selected item.
+            item = self.element.GetNextSelected(item)
+
+        # No selected items.
+        if not len(ids):
+            ids = None
+
+        # Launch the dialog.
+        uf_store['spectrum.error_analysis'](subset=ids, wx_wizard_modal=True)
+
+        # Display the relax controller, and go to the end of the log window.
+        self.gui.show_controller(None)
+        self.gui.controller.log_panel.on_goto_end(None)
+
+
     def action_spectrum_integration_points(self, event):
         """Launch the spectrum.integration_points user function.
 
@@ -430,6 +495,31 @@ class Spectra_list(Base_list):
         return True
 
 
+    def add_offset(self, index):
+        """Add the offset info to the element.
+
+        @param index:   The column index for the data.
+        @type index:    int
+        @return:        True if the data exists, False otherwise.
+        @rtype:         bool
+        """
+
+        # Append a column.
+        self.element.InsertColumn(index, u("Offset \u03C9_rf (ppm)"))
+
+        # No data.
+        if not hasattr(cdp, 'spectrum_ids'):
+            return True
+
+        # Set the values.
+        for i in range(len(cdp.spectrum_ids)):
+            if hasattr(cdp, 'spin_lock_offset') and cdp.spectrum_ids[i] in cdp.spin_lock_offset.keys():
+                self.element.SetStringItem(i, index, float_to_gui(cdp.spin_lock_offset[cdp.spectrum_ids[i]]))
+
+        # Successful.
+        return True
+
+
     def generate_popup_menu(self, id=None):
         """Create the popup menu.
 
@@ -497,6 +587,12 @@ class Spectra_list(Base_list):
                 'icon': fetch_icon("relax.relax_disp"),
                 'method': self.action_relax_disp_spin_lock_field
             })
+            popup_menus.append({
+                'id': wx.NewId(),
+                'text': u("Set the spin-&lock offset \u03C9_rf"),
+                'icon': fetch_icon("relax.relax_disp"),
+                'method': self.action_relax_disp_spin_lock_offset
+            })
         if self.relax_disp_flag and is_cpmg_exp_type(id):
             popup_menus.append({
                 'id': wx.NewId(),
@@ -642,6 +738,7 @@ class Spectra_list(Base_list):
 
         # Button set up.
         self.button_placement = 'top'
+        self.button_size = (170, 40)
         self.button_info = [
             {
                 'object': 'button_add',
@@ -655,6 +752,12 @@ class Spectra_list(Base_list):
                 'icon': fetch_icon('oxygen.actions.list-remove', "22x22"),
                 'method': self.action_spectrum_delete,
                 'tooltip': "Delete loaded relaxation data from the relax data store."
+            }, {
+                'object': 'button_error_analysis',
+                'label': ' Error analysis',
+                'icon': fetch_icon('oxygen.categories.applications-education', "22x22"),
+                'method': self.action_spectrum_error_analysis,
+                'tooltip': "Perform a peak intensity error analysis on the currently loaded data or data subsets.  Select a subset of the spectra below to perform the error analysis only on this subset."
             }
         ]
 
@@ -698,6 +801,10 @@ class Spectra_list(Base_list):
         if self.relax_disp_flag and self.add_disp_point(index):
             index += 1
 
+        # The offset.
+        if self.relax_disp_flag and self.add_offset(index):
+            index += 1
+
         # The relaxation times.
         if (self.relax_fit_flag or self.relax_disp_flag) and self.relax_times(index):
             index += 1
