@@ -1,6 +1,6 @@
 ###############################################################################
 #                                                                             #
-# Copyright (C) 2012-2013 Edward d'Auvergne                                   #
+# Copyright (C) 2012-2014 Edward d'Auvergne                                   #
 #                                                                             #
 # This file is part of the program relax (http://www.nmr-relax.com).          #
 #                                                                             #
@@ -32,6 +32,7 @@ from gui.input_elements.combo_list import Combo_list
 from gui.fonts import font
 from gui.misc import add_border
 from gui.string_conv import float_to_gui, gui_to_float, gui_to_int, gui_to_list, gui_to_py, gui_to_str, gui_to_tuple, int_to_gui, list_to_gui, py_to_gui, str_to_gui, tuple_to_gui
+from lib.check_types import is_list
 from lib.errors import RelaxError
 from status import Status; status = Status()
 
@@ -48,7 +49,7 @@ class Sequence:
         - tuple of strings
     """
 
-    def __init__(self, name=None, default=None, parent=None, element_type='default', seq_type=None, value_type=None, dim=None, min=0, max=1000, sizer=None, desc=None, combo_choices=None, combo_data=None, combo_list_min=None, tooltip=None, divider=None, padding=0, spacer=None, height_element=27, single_value=False, read_only=False, can_be_none=False):
+    def __init__(self, name=None, default=None, parent=None, element_type='default', seq_type=None, value_type=None, dim=None, min=0, max=1000, sizer=None, titles=None, desc=None, combo_choices=None, combo_data=None, combo_list_min=None, tooltip=None, divider=None, padding=0, spacer=None, height_element=27, single_value=False, read_only=False, can_be_none=False):
         """Set up the element.
 
         @keyword name:              The name of the element to use in titles, etc.
@@ -71,6 +72,8 @@ class Sequence:
         @type max:                  int
         @keyword sizer:             The sizer to put the input field widget into.
         @type sizer:                wx.Sizer instance
+        @keyword titles:            The titles of each of the elements of the fixed dimension elements.
+        @type titles:               list of str
         @keyword desc:              The text description.
         @type desc:                 str
         @keyword combo_choices:     The list of choices to present to the user.  This is only used if the element_type is set to 'combo'.
@@ -107,6 +110,7 @@ class Sequence:
         self.dim = dim
         self.min = min
         self.max = max
+        self.titles = titles
         self.single_value = single_value
         self.can_be_none = can_be_none
 
@@ -282,6 +286,14 @@ class Sequence:
                     else:
                         value = ()
 
+                # Handle all other errors.
+                except:
+                    value = None
+
+        # Nothing to do.
+        if value == None:
+            return None
+
         # Convert sequences to single values as needed.
         if self.single_value:
             if (isinstance(value, list) or isinstance(value, tuple)) and len(value) == 1:
@@ -365,7 +377,7 @@ class Sequence:
         value = self.sel_win.GetValue()
 
         # No sequence data.
-        if not len(value):
+        if value == None or not len(value):
             self.Clear()
 
         # Set the values.
@@ -377,7 +389,7 @@ class Sequence:
         """Show the selection window."""
 
         # Initialise the model selection window.
-        self.sel_win = Sequence_window(parent=self.parent, name=self.name, seq_type=self.seq_type, value_type=self.value_type, dim=self.dim)
+        self.sel_win = Sequence_window(parent=self.parent, name=self.name, seq_type=self.seq_type, value_type=self.value_type, titles=self.titles, dim=self.dim)
 
         # Set the model selector window selections.
         self.sel_win.SetValue(self.GetValue())
@@ -437,7 +449,7 @@ class Sequence_window(wx.Dialog):
     # Sizes.
     SIZE_BUTTON = (150, 33)
 
-    def __init__(self, parent=None, name='', seq_type='list', value_type='str', dim=None):
+    def __init__(self, parent=None, name='', seq_type='list', value_type='str', dim=None, titles=None):
         """Set up the string list editor window.
 
         @keyword parent:        The parent GUI element.
@@ -450,6 +462,8 @@ class Sequence_window(wx.Dialog):
         @type value_type:       str
         @keyword dim:           The fixed dimension that the sequence must conform to.
         @type dim:              int or None
+        @keyword titles:        The titles of each of the elements of the fixed dimension elements.
+        @type titles:           list of str
         """
 
         # Store the args.
@@ -457,6 +471,7 @@ class Sequence_window(wx.Dialog):
         self.seq_type = seq_type
         self.value_type = value_type
         self.dim = dim
+        self.titles = titles
 
         # The base types.
         if value_type in ['float', 'num']:
@@ -471,6 +486,14 @@ class Sequence_window(wx.Dialog):
         else:
             raise RelaxError("Unknown base data type '%s'." % value_type)
 
+        # Variable length.
+        if not hasattr(self, 'variable_length'):
+            self.variable_length = False
+            self.offset = 0
+            if dim == None:
+                self.variable_length = True
+                self.offset = 1
+
         # The title of the dialog.
         title = "Edit the %s values." % name
 
@@ -525,12 +548,27 @@ class Sequence_window(wx.Dialog):
             text = item.GetText()
 
             # Store the text.
-            values.append(self.convert_from_gui(text))
+            try:
+                value = self.convert_from_gui(text)
+            except:
+                value = None
+            values.append(value)
 
         # Sequence conversion.
         if self.seq_type == 'tuple':
             values = tuple(values)
 
+        # Check that something is set.
+        empty = True
+        for i in range(len(values)):
+            if values[i] != None:
+                empty = False
+                break
+
+        # Return nothing.
+        if empty:
+            return None
+
         # Return the sequence.
         return values
 
@@ -546,6 +584,10 @@ class Sequence_window(wx.Dialog):
         if values == None:
             return
 
+        # Invalid list, so do nothing.
+        if not self.variable_length and is_list(values) and len(values) != self.dim:
+            return
+
         # Single values.
         try:
             len(values)
@@ -558,7 +600,7 @@ class Sequence_window(wx.Dialog):
         # Loop over the entries.
         for i in range(len(values)):
             # Fixed dimension sequences - set the values of the pre-created list.
-            if self.dim:
+            if not self.variable_length:
                 self.sequence.SetStringItem(index=i, col=1, label=self.convert_to_gui(values[i]))
 
             # Variable dimension sequences - append the item to the end of the blank list.
@@ -584,7 +626,7 @@ class Sequence_window(wx.Dialog):
         sizer.Add(button_sizer, 0, wx.ALIGN_CENTER|wx.ALL, 0)
 
         # The non-fixed sequence buttons.
-        if self.dim == None or (isinstance(self.dim, tuple) and self.dim[0] == None):
+        if self.variable_length:
             # The add button.
             button = wx.lib.buttons.ThemedGenBitmapTextButton(self, -1, None, "  Add")
             button.SetBitmapLabel(wx.Bitmap(fetch_icon('oxygen.actions.list-add-relax-blue', "22x22"), wx.BITMAP_TYPE_ANY))
@@ -640,8 +682,13 @@ class Sequence_window(wx.Dialog):
         # The next index.
         next = self.sequence.GetItemCount()
 
+        # Add a new row with the index at the start.
+        if self.variable_length:
+            self.sequence.InsertStringItem(next, int_to_gui(next+1))
+
         # Add a new empty row.
-        self.sequence.InsertStringItem(next, int_to_gui(next+1))
+        else:
+            self.sequence.InsertStringItem(next, str_to_gui(''))
 
 
     def add_list(self, sizer):
@@ -658,8 +705,12 @@ class Sequence_window(wx.Dialog):
         title = "%s%s" % (self.name[0].upper(), self.name[1:])
 
         # Add the index column.
-        self.sequence.InsertColumn(0, "Position")
-        self.sequence.SetColumnWidth(0, 70)
+        if self.titles:
+            self.sequence.InsertColumn(0, "Title")
+            self.sequence.SetColumnWidth(0, 200)
+        else:
+            self.sequence.InsertColumn(0, "Number")
+            self.sequence.SetColumnWidth(0, 70)
 
         # Add a single column, full width.
         self.sequence.InsertColumn(1, title)
@@ -669,10 +720,19 @@ class Sequence_window(wx.Dialog):
         sizer.Add(self.sequence, 1, wx.ALL|wx.EXPAND, 0)
 
         # The fixed dimension sequence - add all the rows needed.
-        if self.dim:
+        if not self.variable_length:
             for i in range(self.dim):
+                # Add a new row.
                 self.add_element()
 
+                # Add a title to the first column.
+                if self.titles:
+                    self.sequence.SetStringItem(i, 0, str_to_gui(self.titles[i]))
+
+                # Otherwise add numbers starting from 1.
+                else:
+                    self.sequence.SetStringItem(i, 0, int_to_gui(i+1))
+
 
     def close(self, event):
         """Close the window.
@@ -681,8 +741,8 @@ class Sequence_window(wx.Dialog):
         @type event:    wx event
         """
 
-        # Destroy the window.
-        self.Destroy()
+        # Close the window.
+        self.Close()
 
 
     def delete(self, event):
