@@ -1,6 +1,7 @@
 ###############################################################################
 #                                                                             #
-# Copyright (C) 2009-2013 Edward d'Auvergne                                   #
+# Copyright (C) 2009-2014 Edward d'Auvergne                                   #
+# Copyright (C) 2014 Troels E. Linnet                                         #
 #                                                                             #
 # This file is part of the program relax (http://www.nmr-relax.com).          #
 #                                                                             #
@@ -245,6 +246,8 @@ def is_float_object(arg, name=None, dim=(3, 3), can_be_none=False, raise_error=T
 
     # Init.
     fail = False
+    if isinstance(dim, int):
+        dim = [dim]
 
     # An argument of None is allowed.
     if can_be_none and arg == None:
@@ -372,7 +375,7 @@ def is_int(arg, name=None, can_be_none=False, raise_error=True):
         raise RelaxNoneIntError(name, arg)
 
 
-def is_int_list(arg, name=None, size=None, can_be_none=False, can_be_empty=False, none_elements=False, raise_error=True):
+def is_int_list(arg, name=None, size=None, can_be_none=False, can_be_empty=False, none_elements=False, list_of_lists=False, raise_error=True):
     """Test if the argument is a list of integers.
 
     @param arg:                         The argument.
@@ -387,6 +390,8 @@ def is_int_list(arg, name=None, size=None, can_be_none=False, can_be_empty=False
     @type can_be_empty:                 bool
     @keyword none_elements:             A flag which if True allows the list to contain None.
     @type none_elements:                bool
+    @keyword list_of_lists:             A flag which if True allows the argument to be a list of lists.
+    @type list_of_lists:                bool
     @keyword raise_error:               A flag which if True will cause RelaxErrors to be raised.
     @type raise_error:                  bool
     @raise RelaxIntListIntError:        If not an integer or a list of integers (and the raise_error flag is set).
@@ -416,12 +421,18 @@ def is_int_list(arg, name=None, size=None, can_be_none=False, can_be_empty=False
         if not can_be_empty and arg == []:
             fail = True
 
-        # Check the arguments.
+        # Fail if not ints.
         for i in range(len(arg)):
             # None.
             if arg[i] == None and none_elements:
                 continue
 
+            # List of lists.
+            if list_of_lists and isinstance(arg[i], list):
+                for j in range(len(arg[i])):
+                    if not isinstance(arg[i][j], int):
+                        fail = True
+
             # Check if it is an integer.
             if not isinstance(arg[i], int):
                 fail = True
@@ -786,7 +797,7 @@ def is_num_or_num_tuple(arg, name=None, size=None, can_be_none=False, can_be_emp
 
         # Fail if not numbers.
         for i in range(len(arg)):
-            if not is_num(arg[i], raise_error=False):
+            if not (is_num(arg[i], raise_error=False) or (can_be_none and arg[i] == None)):
                 fail = True
 
     # Fail.
@@ -858,9 +869,9 @@ def is_num_tuple(arg, name=None, size=None, can_be_none=False, can_be_empty=Fals
         if not raise_error:
             return False
         if can_be_none and size != None:
-            raise RelaxNoneTupleNumError(name, arg, size)
+            raise RelaxNoneNumTupleNumError(name, arg, size)
         elif can_be_none:
-            raise RelaxNoneTupleNumError(name, arg)
+            raise RelaxNoneNumTupleNumError(name, arg)
         elif size != None:
             raise RelaxTupleNumError(name, arg, size)
         else:
@@ -917,8 +928,7 @@ def is_str_list(arg, name=None, size=None, can_be_none=False, can_be_empty=False
     @type can_be_none:              bool
     @keyword can_be_empty:          A flag which if True allows the list to be empty.
     @type can_be_empty:             bool
-    @keyword list_of_lists:         A flag which if True allows the argument to be a list of lists
-                                    of strings.
+    @keyword list_of_lists:         A flag which if True allows the argument to be a list of lists of strings.
     @type list_of_lists:            bool
     @keyword raise_error:           A flag which if True will cause RelaxErrors to be raised.
     @type raise_error:              bool
@@ -957,7 +967,6 @@ def is_str_list(arg, name=None, size=None, can_be_none=False, can_be_empty=False
                     if not isinstance(arg[i][j], str):
                         fail = True
 
-
             # Simple list.
             else:
                 if not isinstance(arg[i], str):
@@ -1002,7 +1011,7 @@ def is_str_or_inst(arg, name=None, can_be_none=False, raise_error=True):
         return True
 
     # Check for a string.
-    if isinstance(arg, str) or lib.check_types.is_filetype(arg) or isinstance(arg, DummyFileObject):
+    if isinstance(arg, str) or lib.check_types.is_filetype(arg) or isinstance(arg, DummyFileObject) or hasattr(arg, 'write'):
         return True
 
     # Fail.
