@@ -2,6 +2,7 @@ from __future__ import absolute_import
 ###############################################################################
 #                                                                             #
 # Copyright (C) 2003-2014 Edward d'Auvergne                                   #
+# Copyright (C) 2014 Troels E. Linnet                                         #
 #                                                                             #
 # This file is part of the program relax (http://www.nmr-relax.com).          #
 #                                                                             #
@@ -23,8 +24,7 @@ from __future__ import absolute_import
 # Module docstring.
 """Module containing advanced IO functions for relax.
 
-This includes IO redirection, automatic loading and writing of compressed files (both Gzip and BZ2
-compression), reading and writing of files, processing of the contents of files, etc.
+This includes IO redirection, automatic loading and writing of compressed files (both Gzip and BZ2 compression), reading and writing of files, processing of the contents of files, etc.
 """
 
 # Python module imports.
@@ -38,7 +38,7 @@ except ImportError:
 from os import devnull
 from os import F_OK, X_OK, access, altsep, getenv, makedirs, pathsep, remove, sep
 from os.path import expanduser, basename, splitext, isfile
-from re import search
+from re import search, split
 from sys import stdin, stdout, stderr
 from warnings import warn
 
@@ -86,9 +86,7 @@ def determine_compression(file_path):
 
     @param file_path:   The full file path of the file.
     @type file_path:    str
-    @return:            A tuple of the compression type and full path of the file (including its
-                        extension).  A value of 0 corresponds to no compression.  Bzip2 compression
-                        corresponds to a value of 1.  Gzip compression corresponds to a value of 2.
+    @return:            A tuple of the compression type and full path of the file (including its extension).  A value of 0 corresponds to no compression.  Bzip2 compression corresponds to a value of 1.  Gzip compression corresponds to a value of 2.
     @rtype:             (int, str)
     """
 
@@ -123,15 +121,11 @@ def extract_data(file=None, dir=None, file_data=None, sep=None):
 
     @param file:            The file to extract the data from.
     @type file:             str or file object
-    @param dir:             The path where the file is located.  If None and the file argument is a
-                            string, then the current directory is assumed.
+    @param dir:             The path where the file is located.  If None and the file argument is a string, then the current directory is assumed.
     @type dir:              str or None
-    @param file_data:       If the file data has already been extracted from the file, it can be
-                            passed into this function using this argument.  If data is supplied
-                            here, then the file_name and dir args are ignored.
+    @param file_data:       If the file data has already been extracted from the file, it can be passed into this function using this argument.  If data is supplied here, then the file_name and dir args are ignored.
     @type file_data:        list of str
-    @param sep:             The character separating the columns in the file data.  If None, then
-                            whitespace is assumed.
+    @param sep:             The character separating the columns in the file data.  If None, then whitespace is assumed.
     @type sep:              str
     @return:                The file data.
     @rtype:                 list of lists of str
@@ -172,8 +166,13 @@ def file_root(file_path):
     @rtype:             str
     """
 
-    root, ext = splitext(file_path)
-    return basename(root)
+    # Loop over all file extensions, stopping when none are left.
+    ext = None
+    while ext != '':
+        file_path, ext = splitext(file_path)
+
+    # Return the file root with the directories stripped.
+    return basename(file_path)
 
 
 def get_file_path(file_name=None, dir=None):
@@ -181,8 +180,7 @@ def get_file_path(file_name=None, dir=None):
 
     @param file_name:   The name of the file to extract the data from.
     @type file_name:    str
-    @param dir:         The path where the file is located.  If None, then the current directory is
-                        assumed.
+    @param dir:         The path where the file is located.  If None, then the current directory is assumed.
     @type dir:          str
     @return:            The full file path.
     @rtype:             str
@@ -227,8 +225,7 @@ def io_streams_log(file_name=None, dir=None, verbosity=1):
 
     @param file_name:   The name of the file.
     @type file_name:    str
-    @param dir:         The path where the file is located.  If None, then the current directory is
-                        assumed.
+    @param dir:         The path where the file is located.  If None, then the current directory is assumed.
     @type dir:          str
     @param verbosity:   The verbosity level.
     @type verbosity:    int
@@ -263,11 +260,9 @@ def io_streams_tee(file_name=None, dir=None, compress_type=0, verbosity=1):
 
     @param file_name:       The name of the file.
     @type file_name:        str
-    @param dir:             The path where the file is located.  If None, then the current directory
-                            is assumed.
+    @param dir:             The path where the file is located.  If None, then the current directory is assumed.
     @type dir:              str
-    @param compress_type:   The compression type.  The integer values correspond to the compression
-                            type: 0, no compression; 1, Bzip2 compression; 2, Gzip compression.
+    @param compress_type:   The compression type.  The integer values correspond to the compression type: 0, no compression; 1, Bzip2 compression; 2, Gzip compression.
     @type compress_type:    int
     @param verbosity:       The verbosity level.
     @type verbosity:        int
@@ -326,8 +321,7 @@ def open_read_file(file_name=None, dir=None, verbosity=1):
 
     @param file_name:   The name of the file to extract the data from.
     @type file_name:    str
-    @param dir:         The path where the file is located.  If None, then the current directory is
-                        assumed.
+    @param dir:         The path where the file is located.  If None, then the current directory is assumed.
     @type dir:          str
     @param verbosity:   The verbosity level.
     @type verbosity:    int
@@ -382,24 +376,18 @@ def open_write_file(file_name=None, dir=None, force=False, compress_type=0, verb
 
     @param file_name:       The name of the file to extract the data from.
     @type file_name:        str
-    @param dir:             The path where the file is located.  If None, then the current directory
-                            is assumed.
+    @param dir:             The path where the file is located.  If None, then the current directory is assumed.
     @type dir:              str
-    @param force:           Boolean argument which if True causes the file to be overwritten if it
-                            already exists.
+    @param force:           Boolean argument which if True causes the file to be overwritten if it already exists.
     @type force:            bool
-    @param compress_type:   The compression type.  The integer values correspond to the compression
-                            type: 0, no compression; 1, Bzip2 compression; 2, Gzip compression.
+    @param compress_type:   The compression type.  The integer values correspond to the compression type: 0, no compression; 1, Bzip2 compression; 2, Gzip compression.  If no compression is given but the file name ends in '.gz' or '.bz2', then the compression will be automatically set.
     @type compress_type:    int
     @param verbosity:       The verbosity level.
     @type verbosity:        int
-    @param return_path:     If True, the function will return a tuple of the file object and the
-                            full file path.
+    @param return_path:     If True, the function will return a tuple of the file object and the full file path.
     @type return_path:      bool
-    @return:                The open, writable file object and, if the return_path is True, then the
-                            full file path is returned as well.
-    @rtype:                 writable file object (if return_path, then a tuple of the writable file
-                            and the full file path)
+    @return:                The open, writable file object and, if the return_path is True, then the full file path is returned as well.
+    @rtype:                 writable file object (if return_path, then a tuple of the writable file and the full file path)
     """
 
     # No file name?
@@ -437,6 +425,13 @@ def open_write_file(file_name=None, dir=None, force=False, compress_type=0, verb
     # File path.
     file_path = get_file_path(file_name, dir)
 
+    # If no compression is supplied, determine the compression to be used from the file extension.
+    if compress_type == 0:
+        if search('.bz2$', file_path):
+            compress_type = 1
+        elif search('.gz$', file_path):
+            compress_type = 2
+
     # Bzip2 compression.
     if compress_type == 1 and not search('.bz2$', file_path):
         # Bz2 module exists.
@@ -486,6 +481,31 @@ def open_write_file(file_name=None, dir=None, force=False, compress_type=0, verb
         return file_obj
 
 
+def sort_filenames(filenames=None, rev=False):
+    """Sort the given list in alphanumeric order.  Should be equivalent to unix 'ls -n' command.
+
+    @keyword filenames: The list of filenames
+    @type l:            list of strings
+    @keyword rev:       Flag, if the list should be reverted
+    @type rev:          boold
+    """
+
+    # Define function to convert to integers if text is digit.
+    convert = lambda text: int(text) if text.isdigit() else text
+
+    # Define function to create key for sorting.
+    alphanum_key = lambda key: [ convert(c) for c in split('([0-9]+)', key) ]
+
+    # Now sort according to key.
+    filenames.sort( key=alphanum_key )
+
+    # Reverse the list if needed.
+    if rev:
+        return reversed(filenames)
+    else:
+        return filenames
+
+
 def strip(data, comments=True):
     """Remove all comment and empty lines from the file data structure.
 
