@@ -1,7 +1,7 @@
 ###############################################################################
 #                                                                             #
 # Copyright (C) 2003-2014 Edward d'Auvergne                                   #
-# Copyright (C) 2013 Troels E. Linnet                                         #
+# Copyright (C) 2013-2014 Troels E. Linnet                                    #
 #                                                                             #
 # This file is part of the program relax (http://www.nmr-relax.com).          #
 #                                                                             #
@@ -25,7 +25,6 @@
 
 # Python module imports.
 from math import ceil, sqrt
-from re import search
 
 # This script is used to batch convert the Grace *.agr files into graphics files using the Grace
 # program itself.
@@ -178,8 +177,12 @@ def write_xy_data(data, file=None, graph_type=None, norm=None, autoscale=True):
 
                 # The dx and dy errors.
                 if graph_type in ['xydx', 'xydy', 'xydxdy']:
+                    # Catch if graph is a mix with and without error.
+                    if len(point) < 3:
+                        error = None
+                    else:
+                        error = point[2]
                     # Catch x or y-axis errors of None.
-                    error = point[2]
                     if error == None:
                         error = 0.0
 
