@@ -1,6 +1,7 @@
 ###############################################################################
 #                                                                             #
-# Copyright (C) 2003-2013 Edward d'Auvergne                                   #
+# Copyright (C) 2003-2014 Edward d'Auvergne                                   #
+# Copyright (C) 2014 Troels E. Linnet                                         #
 #                                                                             #
 # This file is part of the program relax (http://www.nmr-relax.com).          #
 #                                                                             #
@@ -23,9 +24,6 @@
 """Module for generating OpenDX files."""
 
 
-# Python module imports.
-from numpy import ceil, median
-
 # relax module imports.
 from lib.io import open_write_file
 
