@@ -1,6 +1,6 @@
 ###############################################################################
 #                                                                             #
-# Copyright (C) 2013 Edward d'Auvergne                                        #
+# Copyright (C) 2013-2014 Edward d'Auvergne                                   #
 #                                                                             #
 # This file is part of the program relax (http://www.nmr-relax.com).          #
 #                                                                             #
@@ -84,6 +84,10 @@ def _record_validate(record):
         else:
             raise RelaxError("The PDB record '%s' is too long." % record)
 
+    # Check for nan.
+    if 'nan' in record:
+        raise RelaxError("The PDB record '%s' contains NaN values." % record)
+
 
 def atom(file, serial='', name='', alt_loc='', res_name='', chain_id='', res_seq='', icode='', x='', y='', z='', occupancy='', temp_factor='', element='', charge=''):
     """Generate the ATOM record.
@@ -265,6 +269,16 @@ def atom(file, serial='', name='', alt_loc='', res_name='', chain_id='', res_seq
     @type charge:           int
     """
 
+    # Coordinate bounds.
+    pdb_min = -999.999
+    pdb_max = 9999.999
+    coord = [x, y, z]
+    for i in range(3):
+        if coord[i] != '' and coord[i] < pdb_min:
+            coord[i] = pdb_min
+        if coord[i] != '' and coord[i] > pdb_max:
+            coord[i] = pdb_max
+
     # The formatted record.
     text = "%-6s%5s %-4s%1s%3s %1s%4s%1s   %8.3f%8.3f%8.3f%6.2f%6.2f          %2s%2s" % (
         'ATOM',
@@ -275,9 +289,9 @@ def atom(file, serial='', name='', alt_loc='', res_name='', chain_id='', res_seq
         _handle_none(chain_id),
         _handle_none(res_seq),
         _handle_none(icode),
-        _handle_none(x),
-        _handle_none(y),
-        _handle_none(z),
+        _handle_none(coord[0]),
+        _handle_none(coord[1]),
+        _handle_none(coord[2]),
         _handle_none(occupancy),
         _handle_none(temp_factor),
         _handle_none(element),
@@ -1078,6 +1092,16 @@ def hetatm(file, serial='', name='', alt_loc='', res_name='', chain_id='', res_s
     @type charge:           int
     """
 
+    # Coordinate bounds.
+    pdb_min = -999.999
+    pdb_max = 9999.999
+    coord = [x, y, z]
+    for i in range(3):
+        if coord[i] != '' and coord[i] < pdb_min:
+            coord[i] = pdb_min
+        if coord[i] != '' and coord[i] > pdb_max:
+            coord[i] = pdb_max
+
     # The formatted record.
     text = "%-6s%5s %4s%1s%3s %1s%4s%1s   %8.3f%8.3f%8.3f%6.2f%6.2f          %2s%2s" % (
         'HETATM',
@@ -1088,9 +1112,9 @@ def hetatm(file, serial='', name='', alt_loc='', res_name='', chain_id='', res_s
         _handle_none(chain_id),
         _handle_none(res_seq),
         _handle_none(icode),
-        _handle_none(x),
-        _handle_none(y),
-        _handle_none(z),
+        _handle_none(coord[0]),
+        _handle_none(coord[1]),
+        _handle_none(coord[2]),
         _handle_none(occupancy),
         _handle_none(temp_factor),
         _handle_none(element),
@@ -1504,11 +1528,20 @@ def remark(file, num='', remark=''):
     @type remark:       str
     """
 
-    # The formatted record, splitting across lines if needed.
-    for line in wrap(remark, 68):
-        # The text.
-        text = "%-6s %3s %-68s " % ("REMARK", num, line.upper())
+    # Initialise.
+    lines = []
 
+    # Handle empty lines.
+    if remark == None:
+        lines.append("%-6s %3s %-68s " % ("REMARK", num, ''))
+
+    # The formatted record, splitting across lines if needed.
+    else:
+        for line in wrap(remark, 68):
+            lines.append("%-6s %3s %-68s " % ("REMARK", num, line.upper()))
+
+    # Output all lines.
+    for text in lines:
         # Validate.
         _record_validate(text)
 
