@@ -239,7 +239,7 @@ def grid(func, args=(), num_incs=None, lower=None, upper=None, incs=None, A=None
             # Print out code.
             if verbosity >= 2:
                 if not min_found:
-                    print_iter(k=k, xk=min_params, fk=f, print_prefix=print_prefix)
+                    print_iter(k=k, xk=params, fk=f, print_prefix=print_prefix)
                 if verbosity >= 3:
                     if min_found:
                         print(print_prefix + "Minimum found.")
@@ -365,7 +365,7 @@ def grid_point_array(func, args=(), points=None, A=None, b=None, l=None, u=None,
         # Print out code.
         if verbosity >= 2:
             if f != f_min:
-                print_iter(k=k, xk=min_params, fk=f, print_prefix=print_prefix)
+                print_iter(k=k, xk=params, fk=f, print_prefix=print_prefix)
             if verbosity >= 3:
                 print(print_prefix + "%-20s%-20s" % ("Params:", repr(points[k])))
                 print(print_prefix + "%-20s%-20s" % ("Min params:", repr(min_params)))
@@ -376,34 +376,42 @@ def grid_point_array(func, args=(), points=None, A=None, b=None, l=None, u=None,
     return min_params, f_min, total_steps, None
 
 
-def grid_split(divisions=None, lower=None, upper=None, inc=None, A=None, b=None, l=None, u=None, c=None):
+def grid_split(divisions=None, lower=None, upper=None, inc=None, A=None, b=None, l=None, u=None, c=None, verbosity=0, print_prefix=""):
     """Generator method yielding arrays of grid points.
 
     This method will loop over the grid points one-by-one, generating a list of points and yielding these for each subdivision.
 
 
-    @keyword divisions: The number of grid subdivisions.
-    @type divisions:    int
-    @keyword lower:     The lower bounds of the grid search which must be equal to the number of parameters in the model.
-    @type lower:        array of numbers
-    @keyword upper:     The upper bounds of the grid search which must be equal to the number of parameters in the model.
-    @type upper:        array of numbers
-    @keyword inc:       The increments for each dimension of the space for the grid search.  The number of elements in the array must equal to the number of parameters in the model.
-    @type inc:          array of int
-    @keyword A:         The linear constraint matrix A, such that A.x >= b.
-    @type A:            numpy rank-2 array
-    @keyword b:         The linear constraint scalar vectors, such that A.x >= b.
-    @type b:            numpy rank-1 array
-    @keyword l:         The lower bound constraint vector, such that l <= x <= u.
-    @type l:            list of float
-    @keyword u:         The upper bound constraint vector, such that l <= x <= u.
-    @type u:            list of float
-    @keyword c:         A user supplied constraint function.
-    @type c:            function
-    @return:            A list of grid points for each subdivision is yielded.
-    @rtype:             list of list of float
+    @keyword divisions:     The number of grid subdivisions.
+    @type divisions:        int
+    @keyword lower:         The lower bounds of the grid search which must be equal to the number of parameters in the model.
+    @type lower:            array of numbers
+    @keyword upper:         The upper bounds of the grid search which must be equal to the number of parameters in the model.
+    @type upper:            array of numbers
+    @keyword inc:           The increments for each dimension of the space for the grid search.  The number of elements in the array must equal to the number of parameters in the model.
+    @type inc:              array of int
+    @keyword A:             The linear constraint matrix A, such that A.x >= b.
+    @type A:                numpy rank-2 array
+    @keyword b:             The linear constraint scalar vectors, such that A.x >= b.
+    @type b:                numpy rank-1 array
+    @keyword l:             The lower bound constraint vector, such that l <= x <= u.
+    @type l:                list of float
+    @keyword u:             The upper bound constraint vector, such that l <= x <= u.
+    @type u:                list of float
+    @keyword c:             A user supplied constraint function.
+    @type c:                function
+    @keyword verbosity:     The verbosity level.  0 corresponds to no output, 1 is standard, and higher values cause greater and greater amount of output.
+    @type verbosity:        int
+    @keyword print_prefix:  The text to place before the printed output.
+    @type print_prefix:     str
+    @return:                A list of grid points for each subdivision is yielded.
+    @rtype:                 list of list of float
     """
 
+    # Printout.
+    if verbosity and divisions > 1:
+        print("%sSplitting up the grid points." % print_prefix)
+
     # The dimensionality.
     n = len(inc)
 
@@ -443,8 +451,11 @@ def grid_split(divisions=None, lower=None, upper=None, inc=None, A=None, b=None,
     incs = []
     for k in range(n):
         incs.append([])
-        for i in range(inc[k]):
-            incs[k].append(lower[k] + i * (upper[k] - lower[k]) / (inc[k] - 1))
+        if inc[k] == 1:
+            incs[k].append((lower[k] + upper[k])/2.0)
+        else:
+            for i in range(inc[k]):
+                incs[k].append(lower[k] + i * (upper[k] - lower[k]) / (inc[k] - 1))
 
     # Construct the list of all grid points.
     for i in range(total_pts):
@@ -489,6 +500,12 @@ def grid_split(divisions=None, lower=None, upper=None, inc=None, A=None, b=None,
     if size_float % 1:
         size = size + 1
 
+    # Printout.
+    if verbosity and divisions > 1:
+        print("%s    Total number of grid points:  %20i" % (print_prefix, total_pts))
+        print("%s    Number of divisions:          %20i" % (print_prefix, divisions))
+        print("%s    Subdivision size:             %20i" % (print_prefix, size))
+
     # Subdivide.
     for i in range(min(divisions, total_pts)):
         # The start index.
@@ -501,4 +518,108 @@ def grid_split(divisions=None, lower=None, upper=None, inc=None, A=None, b=None,
             end = total_pts
 
         # Yield the subdivision.
-        yield pts[start: end]
+        yield pts_trimmed[start: end]
+
+
+def grid_split_array(divisions=None, points=None, A=None, b=None, l=None, u=None, c=None, verbosity=0, print_prefix=""):
+    """Generator method yielding arrays of grid points.
+
+    This method will loop over the grid points one-by-one, generating a list of points and yielding these for each subdivision.
+
+
+    @keyword divisions:     The number of grid subdivisions.
+    @type divisions:        int
+    @keyword points:        The array of grid points to split up.
+    @type points:           list of lists or rank-2 numpy array
+    @keyword A:             The linear constraint matrix A, such that A.x >= b.
+    @type A:                numpy rank-2 array
+    @keyword b:             The linear constraint scalar vectors, such that A.x >= b.
+    @type b:                numpy rank-1 array
+    @keyword l:             The lower bound constraint vector, such that l <= x <= u.
+    @type l:                list of float
+    @keyword u:             The upper bound constraint vector, such that l <= x <= u.
+    @type u:                list of float
+    @keyword c:             A user supplied constraint function.
+    @type c:                function
+    @keyword verbosity:     The verbosity level.  0 corresponds to no output, 1 is standard, and higher values cause greater and greater amount of output.
+    @type verbosity:        int
+    @keyword print_prefix:  The text to place before the printed output.
+    @type print_prefix:     str
+    @return:                A list of grid points for each subdivision is yielded.
+    @rtype:                 list of list of float
+    """
+
+    # Printout.
+    if verbosity and divisions > 1:
+        print("%sSplitting up the array of grid points." % print_prefix)
+
+    # Linear constraints.
+    if A != None and b != None:
+        constraint_flag = True
+        constraint_linear = Constraint_linear(A, b)
+        c = constraint_linear.func
+        if verbosity >= 3:
+            print(print_prefix + "Linear constraint matrices.")
+            print(print_prefix + "A: " + repr(A))
+            print(print_prefix + "b: " + repr(b))
+
+    # Bound constraints.
+    elif l != None and u != None:
+        constraint_flag = True
+        raise MinfxError("Bound constraints are not implemented yet.")
+
+    # General constraints.
+    elif c != None:
+        constraint_flag = True
+
+    # No constraints.
+    else:
+        constraint_flag = False
+
+    # Eliminate points outside of constraints.
+    if constraint_flag:
+        # Loop over all points.
+        pts_trimmed = []
+        for i in range(len(points)):
+            # The constraint values.
+            ci = c(points[i])
+
+            # No constraint violations, so store the point.
+            if min(ci) >= 0.0:
+                pts_trimmed.append(points[i])
+
+    # No point elimination.
+    else:
+        pts_trimmed = points
+
+    # Convert to numpy.
+    pts_trimmed = array(pts_trimmed)
+
+    # Total number of points remaining.
+    total_pts = len(pts_trimmed)
+
+    # The subdivision size (round up so the last subdivision is smaller than the rest).
+    size_float = total_pts / float(divisions)
+    size = int(size_float)
+    if size_float % 1:
+        size = size + 1
+
+    # Printout.
+    if verbosity and divisions > 1:
+        print("%s    Total number of grid points:  %20i" % (print_prefix, total_pts))
+        print("%s    Number of divisions:          %20i" % (print_prefix, divisions))
+        print("%s    Subdivision size:             %20i" % (print_prefix, size))
+
+    # Subdivide.
+    for i in range(min(divisions, total_pts)):
+        # The start index.
+        start = i * size
+
+        # The end index.
+        if i != divisions - 1:
+            end = (i + 1) * size
+        else:
+            end = total_pts
+
+        # Yield the subdivision.
+        yield pts_trimmed[start: end]
