@@ -37,6 +37,7 @@ from lib.io import extract_data, strip, write_data
 from lib.warnings import RelaxWarning, RelaxZeroVectorWarning
 from pipe_control import pipes
 from pipe_control.mol_res_spin import Selection, count_spins, exists_mol_res_spin_data, generate_spin_id_unique, return_spin, spin_loop
+from pipe_control.pipes import check_pipe
 
 
 def copy(pipe_from=None, pipe_to=None, spin_id1=None, spin_id2=None, verbose=True):
@@ -63,8 +64,8 @@ def copy(pipe_from=None, pipe_to=None, spin_id1=None, spin_id2=None, verbose=Tru
         pipe_to = pipes.cdp_name()
 
     # Test if the pipe_from and pipe_to data pipes exist.
-    pipes.test(pipe_from)
-    pipes.test(pipe_to)
+    check_pipe(pipe_from)
+    check_pipe(pipe_to)
 
     # Check that the spin IDs exist.
     if spin_id1:
@@ -221,8 +222,12 @@ def define(spin_id1=None, spin_id2=None, pipe=None, direct_bond=False, verbose=T
 
     # Use the structural data to find connected atoms.
     if hasattr(dp, 'structure'):
+        # The selection objects.
+        selection1 = cdp.structure.selection(atom_id=spin_id1)
+        selection2 = cdp.structure.selection(atom_id=spin_id2)
+
         # Loop over the atoms of the first spin selection.
-        for mol_name1, res_num1, res_name1, atom_num1, atom_name1, mol_index1, atom_index1 in dp.structure.atom_loop(atom_id=spin_id1, model_num=1, mol_name_flag=True, res_num_flag=True, res_name_flag=True, atom_num_flag=True, atom_name_flag=True, mol_index_flag=True, index_flag=True):
+        for mol_name1, res_num1, res_name1, atom_num1, atom_name1, mol_index1, atom_index1 in dp.structure.atom_loop(selection=selection1, model_num=1, mol_name_flag=True, res_num_flag=True, res_name_flag=True, atom_num_flag=True, atom_name_flag=True, mol_index_flag=True, index_flag=True):
             # Generate the first spin ID.
             id1 = generate_spin_id_unique(pipe_cont=dp, mol_name=mol_name1, res_num=res_num1, res_name=res_name1, spin_num=atom_num1, spin_name=atom_name1)
 
@@ -231,7 +236,7 @@ def define(spin_id1=None, spin_id2=None, pipe=None, direct_bond=False, verbose=T
                 continue
 
             # Loop over the atoms of the second spin selection.
-            for mol_name2, res_num2, res_name2, atom_num2, atom_name2, mol_index2, atom_index2 in dp.structure.atom_loop(atom_id=spin_id2, model_num=1, mol_name_flag=True, res_num_flag=True, res_name_flag=True, atom_num_flag=True, atom_name_flag=True, mol_index_flag=True, index_flag=True):
+            for mol_name2, res_num2, res_name2, atom_num2, atom_name2, mol_index2, atom_index2 in dp.structure.atom_loop(selection=selection2, model_num=1, mol_name_flag=True, res_num_flag=True, res_name_flag=True, atom_num_flag=True, atom_name_flag=True, mol_index_flag=True, index_flag=True):
                 # Directly bonded atoms.
                 if direct_bond:
                     # Different molecules.
@@ -346,7 +351,7 @@ def exists_data(pipe=None):
         pipe = pipes.cdp_name()
 
     # Test the data pipe.
-    pipes.test(pipe)
+    check_pipe(pipe)
 
     # Get the data pipe.
     dp = pipes.get_pipe(pipe)
@@ -487,7 +492,7 @@ def read_dist(file=None, dir=None, unit='meter', spin_id1_col=None, spin_id2_col
         raise RelaxError("The measurement unit of '%s' must be one of 'meter' or 'Angstrom'." % unit)
 
     # Test if the current data pipe exists.
-    pipes.test()
+    check_pipe()
 
     # Test if sequence data exists.
     if not exists_mol_res_spin_data():
@@ -680,7 +685,7 @@ def unit_vectors(ave=True):
     """
 
     # Test if the current data pipe exists.
-    pipes.test()
+    check_pipe()
 
     # Test if interatomic data exists.
     if not exists_data():
