@@ -28,8 +28,6 @@ import dep_check
 # Python module imports.
 if dep_check.pymol_module:
     import pymol
-from math import pi
-from numpy import float64, transpose, zeros
 from os import F_OK, access, pardir, sep
 PIPE, Popen = None, None
 if dep_check.subprocess_module:
@@ -39,10 +37,9 @@ from time import sleep
 
 # relax module imports.
 from lib.errors import RelaxError, RelaxNoPdbError, RelaxNoSequenceError
-from lib.geometry.rotations import euler_to_R_zyz, R_to_axis_angle
 from lib.io import delete, file_root, get_file_path, open_read_file, open_write_file, test_binary
-from pipe_control import pipes
 from pipe_control.mol_res_spin import exists_mol_res_spin_data
+from pipe_control.pipes import check_pipe
 from pipe_control.result_files import add_result_file
 from specific_analyses.api import return_api
 from status import Status; status = Status()
@@ -227,7 +224,7 @@ def cartoon():
     """Apply the PyMOL cartoon style and colour by secondary structure."""
 
     # Test if the current data pipe exists.
-    pipes.test()
+    check_pipe()
 
     # Test for the structure.
     if not hasattr(cdp, 'structure'):
@@ -362,13 +359,13 @@ def frame_order(ave_pos_file="ave_pos.pdb", rep_file="frame_order.pdb", dist_fil
     """Display the frame order results (the geometric object, average position and distribution).
 
     @keyword ave_pos_file:  The name of the file for the average molecule structure.
-    @type ave_pos_file:     str
+    @type ave_pos_file:     str or None
     @keyword rep_file:      The name of the file of the PDB representation of the frame order dynamics to create.
-    @type rep_file:         str
+    @type rep_file:         str or None
     @keyword dist_file:     The name of the file which will contain multiple models spanning the full dynamics distribution of the frame order model.
-    @type dist_file:        str
+    @type dist_file:        str or None
     @keyword dir:           The name of the directory to place the PDB file into.
-    @type dir:              str
+    @type dir:              str or None
     """
 
     # The path.
@@ -377,9 +374,12 @@ def frame_order(ave_pos_file="ave_pos.pdb", rep_file="frame_order.pdb", dist_fil
         path = dir + sep
 
     # Set up the respective objects.
-    frame_order_ave_pos(file=path+ave_pos_file)
-    frame_order_geometric(file=path+rep_file)
-    frame_order_distribution(file=path+dist_file)
+    if ave_pos_file:
+        frame_order_ave_pos(file=path+ave_pos_file)
+    if rep_file:
+        frame_order_geometric(file=path+rep_file)
+    if dist_file:
+        frame_order_distribution(file=path+dist_file)
 
 
 def frame_order_ave_pos(file=None):
@@ -467,7 +467,7 @@ def macro_apply(data_type=None, style="classic", colour_start_name=None, colour_
     """
 
     # Test if the current data pipe exists.
-    pipes.test()
+    check_pipe()
 
     # Test if sequence data exists.
     if not exists_mol_res_spin_data():
@@ -564,7 +564,7 @@ def macro_write(data_type=None, style="classic", colour_start_name=None, colour_
     """
 
     # Test if the current data pipe exists.
-    pipes.test()
+    check_pipe()
 
     # Test if sequence data exists.
     if not exists_mol_res_spin_data():
@@ -616,7 +616,7 @@ def tensor_pdb(file=None):
     """
 
     # Test if the current data pipe exists.
-    pipes.test()
+    check_pipe()
 
     # Read in the tensor PDB file.
     pymol_obj.exec_cmd("load " + file)
@@ -696,7 +696,7 @@ def vector_dist(file=None):
     """
 
     # Test if the current data pipe exists.
-    pipes.test()
+    check_pipe()
 
     # The file root.
     id = file_root(file)
