@@ -29,7 +29,8 @@ import sys
 from warnings import warn
 
 # relax module imports.
-from lib.errors import RelaxError, RelaxFileError, RelaxNoPdbError, RelaxNoSequenceError
+from lib.checks import Check
+from lib.errors import RelaxError, RelaxFileError
 from lib.io import get_file_path, open_write_file, write_data
 from lib.sequence import write_spin_data
 from lib.structure.internal.displacements import Displacements
@@ -40,7 +41,8 @@ from lib.structure.superimpose import fit_to_first, fit_to_mean
 from lib.warnings import RelaxWarning, RelaxNoPDBFileWarning, RelaxZeroVectorWarning
 from pipe_control import molmol, pipes
 from pipe_control.interatomic import interatomic_loop
-from pipe_control.mol_res_spin import create_spin, exists_mol_res_spin_data, generate_spin_id_unique, linear_ave, return_spin, spin_loop
+from pipe_control.mol_res_spin import check_mol_res_spin_data, create_spin, generate_spin_id_unique, linear_ave, return_spin, spin_loop
+from pipe_control.pipes import cdp_name, check_pipe, get_pipe
 from pipe_control.structure.mass import pipe_centre_of_mass
 from status import Status; status = Status()
 from target_functions.ens_pivot_finder import Pivot_finder
@@ -72,7 +74,7 @@ def add_atom(mol_name=None, atom_name=None, res_name=None, res_num=None, pos=[No
     """
 
     # Test if the current data pipe exists.
-    pipes.test()
+    check_pipe()
 
     # Place the structural object into the relax data store if needed.
     if not hasattr(cdp, 'structure'):
@@ -86,7 +88,7 @@ def add_model(model_num=None):
     """Add a new model to the empty structural data object."""
 
     # Test if the current data pipe exists.
-    pipes.test()
+    check_pipe()
 
     # Place the structural object into the relax data store if needed.
     if not hasattr(cdp, 'structure'):
@@ -101,6 +103,182 @@ def add_model(model_num=None):
     print("Created the empty model number %s." % model_num)
 
 
+def align(pipes=None, models=None, method='fit to mean', atom_id=None, centre_type="centroid", centroid=None):
+    """Superimpose a set of related, but not identical structures.
+
+    @keyword pipes:         The data pipes to include in the alignment and superimposition.
+    @type pipes:            list of str
+    @keyword models:        The list of models to for each data pipe superimpose.  The number of elements must match the pipes argument.  If set to None, then all models will be used.
+    @type models:           list of lists of int or None
+    @keyword method:        The superimposition method.  It must be one of 'fit to mean' or 'fit to first'.
+    @type method:           str
+    @keyword atom_id:       The molecule, residue, and atom identifier string.  This matches the spin ID string format.
+    @type atom_id:          str or None
+    @keyword centre_type:   The type of centre to superimpose over.  This can either be the standard centroid superimposition or the CoM could be used instead.
+    @type centre_type:      str
+    @keyword centroid:      An alternative position of the centroid to allow for different superpositions, for example of pivot point motions.
+    @type centroid:         list of float or numpy rank-1, 3D array
+    """
+
+    # Check the method.
+    allowed = ['fit to mean', 'fit to first']
+    if method not in allowed:
+        raise RelaxError("The superimposition method '%s' is unknown.  It must be one of %s." % (method, allowed))
+
+    # Check the type.
+    allowed = ['centroid', 'CoM']
+    if centre_type not in allowed:
+        raise RelaxError("The superimposition centre type '%s' is unknown.  It must be one of %s." % (centre_type, allowed))
+
+    # The data pipes to use.
+    if pipes == None:
+        pipes = [pipes.cdp_name()]
+
+    # Checks.
+    for pipe in pipes:
+        check_pipe(pipe)
+
+    # Initialise the models data structure.
+    if models == None:
+        models = []
+        for i in range(len(pipes)):
+            models.append(None)
+
+    # Assemble the atomic coordinates of all structures.
+    print("Assembling all atomic coordinates:")
+    atom_ids = []
+    atom_pos = []
+    atom_elem = []
+    mol_ids = []
+    for pipe_index in range(len(pipes)):
+        # Printout.
+        print("    Data pipe: %s" % pipes[pipe_index])
+
+        # The data pipe object.
+        dp = get_pipe(pipes[pipe_index])
+
+        # Validate the models.
+        dp.structure.validate_models(verbosity=0)
+
+        # The selection object.
+        selection = dp.structure.selection(atom_id=atom_id)
+
+        # Create a list of all models for this pipe.
+        if models[pipe_index] == None:
+            models[pipe_index] = []
+            for model in dp.structure.model_loop():
+                models[pipe_index].append(model.num)
+
+        # Loop over the models.
+        for model in models[pipe_index]:
+            # Printout.
+            print("        Model: %s" % model)
+
+            # Extend the lists.
+            atom_ids.append([])
+            atom_pos.append({})
+            atom_elem.append({})
+            mol_ids.append([])
+
+            # Add all coordinates and elements.
+            for mol_name, res_num, res_name, atom_name, elem, pos in dp.structure.atom_loop(selection=selection, model_num=model, mol_name_flag=True, res_num_flag=True, res_name_flag=True, atom_name_flag=True, pos_flag=True, element_flag=True):
+                # A unique identifier.
+                id = "#%s:%s&%s@%s" % (mol_name, res_num, res_name, atom_name)
+
+                atom_ids[-1].append(id)
+                atom_pos[-1][id] = pos[0]
+                atom_elem[-1][id] = elem
+
+                # Store the molecule name for later checks.
+                if mol_name not in mol_ids[-1]:
+                    print("            Molecule: %s" % mol_name)
+                    mol_ids[-1].append(mol_name)
+
+    # Check for the molecule names.
+    for mol_name in mol_ids[0]:
+        for i in range(len(mol_ids)):
+            if mol_name not in mol_ids[i]:
+                raise RelaxError("The molecule name '%s' cannot be found in all data pipes." % mol_name)
+
+    # Set up the structures for the superimposition algorithm.
+    num = len(atom_ids)
+    coord = []
+    elements = []
+    for i in range(num):
+        coord.append([])
+        elements.append([])
+
+    # Find the common atoms and create the coordinate data structure.
+    for id in atom_ids[0]:
+        # Is the atom ID present in all other structures?
+        present = True
+        for i in range(num):
+            if id not in atom_ids[i]:
+                present = False
+                break
+
+        # Not present, so skip the atom.
+        if not present:
+            continue
+
+        # Add the atomic position to the coordinate list and the element to the element list.
+        for i in range(num):
+            coord[i].append(atom_pos[i][id])
+            elements[i].append(atom_elem[i][id])
+
+    # Convert to a numpy array.
+    coord = array(coord, float64)
+
+    # The different algorithms.
+    if method == 'fit to mean':
+        T, R, pivot = fit_to_mean(models=range(num), coord=coord, centre_type=centre_type, elements=elements, centroid=centroid)
+    elif method == 'fit to first':
+        T, R, pivot = fit_to_first(models=range(num), coord=coord, centre_type=centre_type, elements=elements, centroid=centroid)
+
+    # Update to the new coordinates.
+    i = 0
+    for pipe_index in range(len(pipes)):
+        for model in models[pipe_index]:
+            # Translate the molecule first (the rotational pivot is defined in the first model).
+            translate(T=T[i], model=model, pipe_name=pipes[pipe_index])
+
+            # Rotate the molecule.
+            rotate(R=R[i], origin=pivot[i], model=model, pipe_name=pipes[pipe_index])
+
+            # Increment the index.
+            i += 1
+
+
+def check_structure_func(pipe_name=None):
+    """Test if structural data is present.
+
+    @return:        The initialised RelaxError object or nothing.
+    @rtype:         None or RelaxError instance
+    """
+
+    # Defaults.
+    if pipe_name == None:
+        pipe_name = cdp_name()
+
+    # Get the data pipe.
+    dp = get_pipe(pipe_name)
+
+    # Test if the structure exists.
+    if not hasattr(dp, 'structure'):
+        return RelaxError("No structural data is present in the current data pipe.")
+
+    # Check for models:
+    if not dp.structure.num_models():
+        return RelaxError("The structural object in the current data pipe contains no models.")
+
+    # Check for molecules.
+    if not dp.structure.num_molecules():
+        return RelaxError("The structural object in the current data pipe contains no molecules.")
+
+# Create the checking object.
+check_structure = Check(check_structure_func)
+
+
 def connect_atom(index1=None, index2=None):
     """Connect two atoms.
 
@@ -111,7 +289,7 @@ def connect_atom(index1=None, index2=None):
     """
 
     # Test if the current data pipe exists.
-    pipes.test()
+    check_pipe()
 
     # Place the structural object into the relax data store if needed.
     if not hasattr(cdp, 'structure'):
@@ -121,63 +299,23 @@ def connect_atom(index1=None, index2=None):
     cdp.structure.connect_atom(index1=index1, index2=index2)
 
 
-def create_cone_pdb(mol=None, cone=None, start_res=1, apex=None, axis=None, R=None, inc=None, scale=30.0, distribution='regular', file=None, dir=None, force=False, axis_flag=True):
-    """Create a PDB representation of the given cone object.
+def com(model=None, atom_id=None):
+    """Calculate the centre of mass (CoM) of all structures.
 
-    @keyword mol:           The molecule container.
-    @type mol:              MolContainer instance
-    @keyword cone:          The cone object.  This should provide the limit_check() method with determines the limits of the distribution accepting two arguments, the polar angle phi and the azimuthal angle theta, and return True if the point is in the limits or False if outside.  It should also provide the theta_max() method for returning the theta value for the given phi, the phi_max() method for returning the phi value for the given theta.
-    @type cone:             class instance
-    @keyword start_res:     The starting residue number.
-    @type start_res:        str
-    @keyword apex:          The apex of the cone.
-    @type apex:             rank-1, 3D numpy array
-    @keyword axis:          The central axis of the cone.  If not supplied, the z-axis will be used.
-    @type axis:             rank-1, 3D numpy array
-    @keyword R:             The rotation matrix.
-    @type R:                rank-2, 3D numpy array
-    @keyword inc:           The increment number used to determine the number of latitude and longitude lines.
-    @type inc:              int
-    @keyword scale:         The scaling factor to stretch the unit cone by.
-    @type scale:            float
-    @keyword distribution:  The type of point distribution to use.  This can be 'uniform' or 'regular'.
-    @type distribution:     str
-    @keyword file:          The name of the PDB file to create.
-    @type file:             str
-    @keyword dir:           The name of the directory to place the PDB file into.
-    @type dir:              str
-    @keyword force:         Flag which if set to True will overwrite any pre-existing file.
-    @type force:            bool
-    @keyword axis_flag:     A flag which if True will create the cone's axis.
-    @type axis_flag:        bool
+    The value will be stored in the current data pipe 'com' variable.
+
+
+    @keyword model:     Only use a specific model.
+    @type model:        int or None
+    @keyword atom_id:   The molecule, residue, and atom identifier string.  This matches the spin ID string format.  If not given, then all structural data will be used for calculating the CoM.
+    @type atom_id:      str or None
     """
 
-    # No molecule supplied.
-    if mol == None:
-        # Create the structural object.
-        structure = Internal()
+    # Test if the current data pipe exists.
+    check_pipe()
 
-        # Add a molecule.
-        structure.add_molecule(name='cone')
-
-        # Alias the single molecule from the single model.
-        mol = structure.structural_data[0].mol[0]
-
-    # Create the object.
-    cone(mol=mol, cone=cone, start_res=start_res, apex=apex, axis=axis, R=R, inc=inc, scale=scale, distribution=distribution, axis_flag=axis_flag)
-
-    # Create the PDB file.
-    if file != None:
-        print("\nGenerating the PDB file.")
-        pdb_file = open_write_file(file_name=file, dir=dir, force=force)
-        structure.write_pdb(pdb_file)
-        pdb_file.close()
-
-    # Add the file to the results file list.
-    if not hasattr(cdp, 'result_files'):
-        cdp.result_files = []
-    cdp.result_files.append(['cone_pdb', 'Cone PDB', get_file_path(file, dir)])
-    status.observers.result_file.notify()
+    # Calculate and store the centre of mass.
+    cdp.com = pipe_centre_of_mass(model=model, atom_id=atom_id)
 
 
 def create_diff_tensor_pdb(scale=1.8e-6, file=None, dir=None, force=False):
@@ -194,10 +332,13 @@ def create_diff_tensor_pdb(scale=1.8e-6, file=None, dir=None, force=False):
     """
 
     # Test if the current data pipe exists.
-    pipes.test()
+    check_pipe()
 
     # Calculate the centre of mass.
-    com = pipe_centre_of_mass()
+    if hasattr(cdp, 'structure') and not cdp.structure.empty():
+        com = pipe_centre_of_mass()
+    else:
+        com = zeros(3, float64)
 
     # Create the structural object.
     structure = Internal()
@@ -225,10 +366,6 @@ def create_diff_tensor_pdb(scale=1.8e-6, file=None, dir=None, force=False):
         if not hasattr(pipe, 'diff_tensor'):
             raise RelaxNoTensorError('diffusion')
 
-        # Test if a structure has been loaded.
-        if not hasattr(cdp, 'structure'):
-            raise RelaxNoPdbError
-
         # Add a new structure.
         structure.add_molecule(name=mol_names[pipe_index])
 
@@ -293,11 +430,13 @@ def create_diff_tensor_pdb(scale=1.8e-6, file=None, dir=None, force=False):
     status.observers.result_file.notify()
 
 
-def delete(atom_id=None, verbosity=1, spin_info=True):
+def delete(atom_id=None, model=None, verbosity=1, spin_info=True):
     """Delete structural data.
     
     @keyword atom_id:   The molecule, residue, and atom identifier string.  This matches the spin ID string format.  If not given, then all structural data will be deleted.
     @type atom_id:      str or None
+    @keyword model:     Individual structural models from a loaded ensemble can be deleted by specifying the model number.
+    @type model:        None or int
     @keyword verbosity: The amount of information to print to screen.  Zero corresponds to minimal output while higher values increase the amount of output.  The default value is 1.
     @type verbosity:    int
     @keyword spin_info: A flag which if True will cause all structural information in the spin containers and interatomic data containers to be deleted as well.  If False, then only the 3D structural data will be deleted.
@@ -305,13 +444,16 @@ def delete(atom_id=None, verbosity=1, spin_info=True):
     """
 
     # Test if the current data pipe exists.
-    pipes.test()
+    check_pipe()
 
     # Run the object method.
     if hasattr(cdp, 'structure'):
         if verbosity:
             print("Deleting structural data from the current pipe.")
-        cdp.structure.delete(atom_id=atom_id, verbosity=verbosity)
+        selection = None
+        if atom_id != None:
+            selection = cdp.structure.selection(atom_id=atom_id)
+        cdp.structure.delete(model=model, selection=selection, verbosity=verbosity)
     elif verbosity:
         print("No structures are present.")
 
@@ -350,7 +492,7 @@ def displacement(model_from=None, model_to=None, atom_id=None, centroid=None):
     """
 
     # Test if the current data pipe exists.
-    pipes.test()
+    check_pipe()
 
     # Convert the model_from and model_to args to lists, is supplied.
     if model_from != None:
@@ -373,18 +515,21 @@ def displacement(model_from=None, model_to=None, atom_id=None, centroid=None):
     if not hasattr(cdp.structure, 'displacements'):
         cdp.structure.displacements = Displacements()
 
+    # The selection object.
+    selection = cdp.structure.selection(atom_id=atom_id)
+
     # Loop over the starting models.
     for i in range(len(model_from)):
         # Assemble the atomic coordinates.
         coord_from = []
-        for pos in cdp.structure.atom_loop(atom_id=atom_id, model_num=model_from[i], pos_flag=True):
+        for pos in cdp.structure.atom_loop(selection=selection, model_num=model_from[i], pos_flag=True):
             coord_from.append(pos[0])
 
         # Loop over the ending models.
         for j in range(len(model_to)):
             # Assemble the atomic coordinates.
             coord_to = []
-            for pos in cdp.structure.atom_loop(atom_id=atom_id, model_num=model_to[j], pos_flag=True):
+            for pos in cdp.structure.atom_loop(selection=selection, model_num=model_to[j], pos_flag=True):
                 coord_to.append(pos[0])
 
             # Send to the base container for the calculations.
@@ -407,7 +552,7 @@ def find_pivot(models=None, atom_id=None, init_pos=None, func_tol=1e-5, box_limi
     """
 
     # Test if the current data pipe exists.
-    pipes.test()
+    check_pipe()
 
     # Initialised the starting position if needed.
     if init_pos == None:
@@ -423,11 +568,14 @@ def find_pivot(models=None, atom_id=None, init_pos=None, func_tol=1e-5, box_limi
         for model in cdp.structure.model_loop():
             models.append(model.num)
 
+    # The selection object.
+    selection = cdp.structure.selection(atom_id=atom_id)
+
     # Assemble the atomic coordinates of all models.
     coord = []
     for model in models:
         coord.append([])
-        for pos in cdp.structure.atom_loop(atom_id=atom_id, model_num=model, pos_flag=True):
+        for pos in cdp.structure.atom_loop(selection=selection, model_num=model, pos_flag=True):
             coord[-1].append(pos[0])
         coord[-1] = array(coord[-1])
     coord = array(coord)
@@ -467,16 +615,16 @@ def get_pos(spin_id=None, str_id=None, ave_pos=False):
     @type ave_pos:              bool
     """
 
-    # Test if the current data pipe exists.
-    pipes.test()
+    # Checks.
+    check_pipe()
+    check_structure()
 
-    # Test if the structure exists.
-    if not hasattr(cdp, 'structure') or not cdp.structure.num_models() or not cdp.structure.num_molecules():
-        raise RelaxNoPdbError
+    # The selection object.
+    selection = cdp.structure.selection(atom_id=spin_id)
 
     # Loop over all atoms of the spin_id selection.
     data = []
-    for mol_name, res_num, res_name, atom_num, atom_name, element, pos in cdp.structure.atom_loop(atom_id=spin_id, str_id=str_id, mol_name_flag=True, res_num_flag=True, res_name_flag=True, atom_num_flag=True, atom_name_flag=True, element_flag=True, pos_flag=True, ave=ave_pos):
+    for mol_name, res_num, res_name, atom_num, atom_name, element, pos in cdp.structure.atom_loop(selection=selection, str_id=str_id, mol_name_flag=True, res_num_flag=True, res_name_flag=True, atom_num_flag=True, atom_name_flag=True, element_flag=True, pos_flag=True, ave=ave_pos):
         # Remove the '+' regular expression character from the mol, res, and spin names!
         if mol_name and search('\+', mol_name):
             mol_name = mol_name.replace('+', '')
@@ -564,12 +712,9 @@ def load_spins(spin_id=None, str_id=None, mol_name_target=None, ave_pos=False):
     @type ave_pos:              bool
     """
 
-    # Test if the current data pipe exists.
-    pipes.test()
-
-    # Test if the structure exists.
-    if not hasattr(cdp, 'structure') or not cdp.structure.num_models() or not cdp.structure.num_molecules():
-        raise RelaxNoPdbError
+    # Checks.
+    check_pipe()
+    check_structure()
 
     # Print out.
     print("Adding the following spins to the relax data store.\n")
@@ -582,7 +727,8 @@ def load_spins(spin_id=None, str_id=None, mol_name_target=None, ave_pos=False):
     spin_names = []
 
     # Loop over all atoms of the spin_id selection.
-    for mol_name, res_num, res_name, atom_num, atom_name, element, pos in cdp.structure.atom_loop(atom_id=spin_id, str_id=str_id, mol_name_flag=True, res_num_flag=True, res_name_flag=True, atom_num_flag=True, atom_name_flag=True, element_flag=True, pos_flag=True, ave=ave_pos):
+    selection = cdp.structure.selection(atom_id=spin_id)
+    for mol_name, res_num, res_name, atom_num, atom_name, element, pos in cdp.structure.atom_loop(selection=selection, str_id=str_id, mol_name_flag=True, res_num_flag=True, res_name_flag=True, atom_num_flag=True, atom_name_flag=True, element_flag=True, pos_flag=True, ave=ave_pos):
         # Override the molecule name.
         if mol_name_target:
             mol_name = mol_name_target
@@ -631,6 +777,17 @@ def load_spins(spin_id=None, str_id=None, mol_name_target=None, ave_pos=False):
     write_spin_data(file=sys.stdout, mol_names=mol_names, res_nums=res_nums, res_names=res_names, spin_nums=spin_nums, spin_names=spin_names)
 
 
+def mean():
+    """Calculate the mean structure from all models in the structural data object."""
+
+    # Checks.
+    check_pipe()
+    check_structure()
+
+    # Call the specific code.
+    cdp.structure.mean()
+
+
 def read_gaussian(file=None, dir=None, set_mol_name=None, set_model_num=None, verbosity=1, fail=True):
     """Read structures from a Gaussian log file.
 
@@ -651,7 +808,7 @@ def read_gaussian(file=None, dir=None, set_mol_name=None, set_model_num=None, ve
     """
 
     # Test if the current data pipe exists.
-    pipes.test()
+    check_pipe()
 
     # The file path.
     file_path = get_file_path(file, dir)
@@ -704,7 +861,7 @@ def read_pdb(file=None, dir=None, read_mol=None, set_mol_name=None, read_model=N
     """
 
     # Test if the current data pipe exists.
-    pipes.test()
+    check_pipe()
 
     # The file path.
     file_path = get_file_path(file, dir)
@@ -772,7 +929,7 @@ def read_xyz(file=None, dir=None, read_mol=None, set_mol_name=None, read_model=N
     """
 
     # Test if the current data pipe exists.
-    pipes.test()
+    check_pipe()
 
     # The file path.
     file_path = get_file_path(file, dir)
@@ -810,7 +967,7 @@ def rmsd(atom_id=None, models=None):
     """
 
     # Test if the current data pipe exists.
-    pipes.test()
+    check_pipe()
 
     # Create a list of all models.
     if models == None:
@@ -818,11 +975,14 @@ def rmsd(atom_id=None, models=None):
         for model in cdp.structure.model_loop():
             models.append(model.num)
 
+    # The selection object.
+    selection = cdp.structure.selection(atom_id=atom_id)
+
     # Assemble the atomic coordinates of all models.
     coord = []
     for model in models:
         coord.append([])
-        for pos in cdp.structure.atom_loop(atom_id=atom_id, model_num=model, pos_flag=True):
+        for pos in cdp.structure.atom_loop(selection=selection, model_num=model, pos_flag=True):
             coord[-1].append(pos[0])
         coord[-1] = array(coord[-1])
 
@@ -833,7 +993,7 @@ def rmsd(atom_id=None, models=None):
     return cdp.structure.rmsd
 
 
-def rotate(R=None, origin=None, model=None, atom_id=None):
+def rotate(R=None, origin=None, model=None, atom_id=None, pipe_name=None):
     """Rotate the structural data about the origin by the specified forwards rotation.
 
     @keyword R:         The forwards rotation matrix.
@@ -844,14 +1004,20 @@ def rotate(R=None, origin=None, model=None, atom_id=None):
     @type model:        int
     @keyword atom_id:   The molecule, residue, and atom identifier string.  Only atoms matching this selection will be used.
     @type atom_id:      str or None
+    @keyword pipe_name: The name of the data pipe containing the structures to translate.  This defaults to the current data pipe.
+    @type pipe_name:    None or str
     """
 
-    # Test if the current data pipe exists.
-    pipes.test()
+    # Defaults.
+    if pipe_name == None:
+        pipe_name = cdp_name()
 
-    # Test if the structure exists.
-    if not hasattr(cdp, 'structure') or not cdp.structure.num_models() or not cdp.structure.num_molecules():
-        raise RelaxNoPdbError
+    # Checks.
+    check_pipe(pipe_name)
+    check_structure(pipe_name)
+
+    # Get the data pipe.
+    dp = get_pipe(pipe_name)
 
     # Set the origin if not supplied.
     if origin == None:
@@ -862,7 +1028,8 @@ def rotate(R=None, origin=None, model=None, atom_id=None):
     origin = array(origin, float64)
 
     # Call the specific code.
-    cdp.structure.rotate(R=R, origin=origin, model=model, atom_id=atom_id)
+    selection = dp.structure.selection(atom_id=atom_id)
+    dp.structure.rotate(R=R, origin=origin, model=model, selection=selection)
 
 
 def set_vector(spin=None, xh_vect=None):
@@ -904,7 +1071,7 @@ def superimpose(models=None, method='fit to mean', atom_id=None, centre_type="ce
         raise RelaxError("The superimposition centre type '%s' is unknown.  It must be one of %s." % (centre_type, allowed))
 
     # Test if the current data pipe exists.
-    pipes.test()
+    check_pipe()
 
     # Validate the models.
     cdp.structure.validate_models()
@@ -915,17 +1082,20 @@ def superimpose(models=None, method='fit to mean', atom_id=None, centre_type="ce
         for model in cdp.structure.model_loop():
             models.append(model.num)
 
+    # The selection object.
+    selection = cdp.structure.selection(atom_id=atom_id)
+
     # Assemble the atomic coordinates of all models.
     coord = []
     for model in models:
         coord.append([])
-        for pos in cdp.structure.atom_loop(atom_id=atom_id, model_num=model, pos_flag=True):
+        for pos in cdp.structure.atom_loop(selection=selection, model_num=model, pos_flag=True):
             coord[-1].append(pos[0])
         coord[-1] = array(coord[-1])
 
     # Assemble the element types.
     elements = []
-    for elem in cdp.structure.atom_loop(atom_id=atom_id, model_num=model, element_flag=True):
+    for elem in cdp.structure.atom_loop(selection=selection, model_num=model, element_flag=True):
         elements.append(elem)
 
     # The different algorithms.
@@ -934,7 +1104,6 @@ def superimpose(models=None, method='fit to mean', atom_id=None, centre_type="ce
     elif method == 'fit to first':
         T, R, pivot = fit_to_first(models=models, coord=coord, centre_type=centre_type, elements=elements, centroid=centroid)
 
-
     # Update to the new coordinates.
     for i in range(len(models)):
         # Translate the molecule first (the rotational pivot is defined in the first model).
@@ -944,7 +1113,7 @@ def superimpose(models=None, method='fit to mean', atom_id=None, centre_type="ce
         rotate(R=R[i], origin=pivot[i], model=models[i])
 
 
-def translate(T=None, model=None, atom_id=None):
+def translate(T=None, model=None, atom_id=None, pipe_name=None):
     """Shift the structural data by the specified translation vector.
 
     @keyword T:         The translation vector.
@@ -953,20 +1122,27 @@ def translate(T=None, model=None, atom_id=None):
     @type model:        int or None
     @keyword atom_id:   The molecule, residue, and atom identifier string.  Only atoms matching this selection will be used.
     @type atom_id:      str or None
+    @keyword pipe_name: The name of the data pipe containing the structures to translate.  This defaults to the current data pipe.
+    @type pipe_name:    None or str
     """
 
-    # Test if the current data pipe exists.
-    pipes.test()
+    # Defaults.
+    if pipe_name == None:
+        pipe_name = cdp_name()
 
-    # Test if the structure exists.
-    if not hasattr(cdp, 'structure') or not cdp.structure.num_models() or not cdp.structure.num_molecules():
-        raise RelaxNoPdbError
+    # Checks.
+    check_pipe(pipe_name)
+    check_structure(pipe_name)
+
+    # Get the data pipe.
+    dp = get_pipe(pipe_name)
 
     # Convert the args to numpy float data structures.
     T = array(T, float64)
 
     # Call the specific code.
-    cdp.structure.translate(T=T, model=model, atom_id=atom_id)
+    selection = dp.structure.selection(atom_id=atom_id)
+    dp.structure.translate(T=T, model=model, selection=selection)
 
 
 def vectors(spin_id1=None, spin_id2=None, model=None, verbosity=1, ave=True, unit=True):
@@ -986,16 +1162,10 @@ def vectors(spin_id1=None, spin_id2=None, model=None, verbosity=1, ave=True, uni
     @type unit:             bool
     """
 
-    # Test if the current data pipe exists.
-    pipes.test()
-
-    # Test if the PDB file has been loaded.
-    if not hasattr(cdp, 'structure'):
-        raise RelaxNoPdbError
-
-    # Test if sequence data is loaded.
-    if not exists_mol_res_spin_data():
-        raise RelaxNoSequenceError
+    # Checks.
+    check_pipe()
+    check_structure()
+    check_mol_res_spin_data()
 
     # Print out.
     if verbosity:
@@ -1132,12 +1302,9 @@ def web_of_motion(file=None, dir=None, models=None, force=False):
     @type force:            bool
     """
 
-    # Test if the current data pipe exists.
-    pipes.test()
-
-    # Test if the structure exists.
-    if not hasattr(cdp, 'structure') or not cdp.structure.num_models() or not cdp.structure.num_molecules():
-        raise RelaxNoPdbError
+    # Checks.
+    check_pipe()
+    check_structure()
 
     # Validate the models.
     cdp.structure.validate_models()
@@ -1216,7 +1383,7 @@ def write_pdb(file=None, dir=None, model_num=None, compress_type=0, force=False)
     """
 
     # Test if the current data pipe exists.
-    pipes.test()
+    check_pipe()
 
     # Check if the structural object exists.
     if not hasattr(cdp, 'structure'):
