@@ -1,6 +1,7 @@
 ###############################################################################
 #                                                                             #
 # Copyright (C) 2003-2014 Edward d'Auvergne                                   #
+# Copyright (C) 2014 Troels E. Linnet                                         #
 #                                                                             #
 # This file is part of the program relax (http://www.nmr-relax.com).          #
 #                                                                             #
@@ -33,12 +34,13 @@ from lib.io import get_file_path, open_write_file
 from lib.sequence import read_spin_data, write_spin_data
 from pipe_control import minimise, pipes
 from pipe_control.mol_res_spin import exists_mol_res_spin_data, generate_spin_id_unique, spin_loop
+from pipe_control.pipes import check_pipe
 from pipe_control.result_files import add_result_file
 from specific_analyses.api import return_api
 from status import Status; status = Status()
 
 
-def copy(pipe_from=None, pipe_to=None, param=None):
+def copy(pipe_from=None, pipe_to=None, param=None, force=False):
     """Copy spin specific data values from pipe_from to pipe_to.
 
     @param pipe_from:   The data pipe to copy the value from.  This defaults to the current data
@@ -48,6 +50,8 @@ def copy(pipe_from=None, pipe_to=None, param=None):
     @type pipe_to:      str
     @param param:       The name of the parameter to copy the values of.
     @type param:        str
+    @keyword force:     A flag forcing the overwriting of current values.
+    @type force:        bool
     """
 
     # The current data pipe.
@@ -58,7 +62,7 @@ def copy(pipe_from=None, pipe_to=None, param=None):
     pipe_orig = pipes.cdp_name()
 
     # The second pipe does not exist.
-    pipes.test(pipe_to)
+    check_pipe(pipe_to)
 
     # Test if the sequence data for pipe_from is loaded.
     if not exists_mol_res_spin_data(pipe_from):
@@ -72,13 +76,14 @@ def copy(pipe_from=None, pipe_to=None, param=None):
     api = return_api(pipe_name=pipe_from)
 
     # Test if the data exists for pipe_to.
-    for spin in spin_loop(pipe=pipe_to):
-        # Get the value and error for pipe_to.
-        value, error = api.return_value(spin, param)
+    if force == False:
+        for spin in spin_loop(pipe=pipe_to):
+            # Get the value and error for pipe_to.
+            value, error = api.return_value(spin, param)
 
-        # Data exists.
-        if value != None or error != None:
-            raise RelaxValueError(param, pipe_to)
+            # Data exists.
+            if value != None or error != None:
+                raise RelaxValueError(param, pipe_to)
 
     # Switch to the data pipe to copy values to.
     pipes.switch(pipe_to)
@@ -90,9 +95,9 @@ def copy(pipe_from=None, pipe_to=None, param=None):
 
         # Set the values of pipe_to.
         if value != None:
-            set(spin_id=spin_id, val=value, param=param, pipe=pipe_to)
+            set(spin_id=spin_id, val=value, param=param, pipe=pipe_to, force=force)
         if error != None:
-            set(spin_id=spin_id, val=error, param=param, pipe=pipe_to, error=True)
+            set(spin_id=spin_id, val=error, param=param, pipe=pipe_to, error=True, force=force)
 
     # Reset all minimisation statistics.
     minimise.reset_min_stats(pipe_to)
@@ -111,7 +116,7 @@ def display(param=None, scaling=1.0):
     """
 
     # Test if the current pipe exists.
-    pipes.test()
+    check_pipe()
 
     # Test if the sequence data is loaded.
     if not exists_mol_res_spin_data():
@@ -277,7 +282,7 @@ def read(param=None, scaling=1.0, file=None, dir=None, file_data=None, spin_id_c
     """
 
     # Test if the current pipe exists.
-    pipes.test()
+    check_pipe()
 
     # Test if sequence data is loaded.
     if not exists_mol_res_spin_data():
@@ -373,7 +378,7 @@ def set(val=None, param=None, index=None, pipe=None, spin_id=None, error=False,
         pipes.switch(pipe)
 
     # Test if the current data pipe exists.
-    pipes.test()
+    check_pipe()
 
     # The specific analysis API object.
     api = return_api()
@@ -451,7 +456,7 @@ def write(param=None, file=None, dir=None, scaling=1.0, return_value=None, retur
     """
 
     # Test if the current pipe exists.
-    pipes.test()
+    check_pipe()
 
     # Test if the sequence data is loaded.
     if not exists_mol_res_spin_data():
@@ -644,6 +649,20 @@ def write_data(param=None, file=None, scaling=1.0, bc=False, return_value=None,
             values.append(scale(value, scaling))
             errors.append(scale(error, scaling))
 
+    # Replace all spaces " " with "_" in data_names list.
+    if isinstance(data_names, list):
+        for i, data_name in enumerate(data_names):
+            data_str = data_name.replace(" ", "_")
+            # Replace string.
+            data_names[i] = data_str
+
+    # Replace all spaces " " with "_" in error_names list.
+    if isinstance(error_names, list):
+        for i, error_name in enumerate(error_names):
+            error_str = error_name.replace(" ", "_")
+            # Replace string.
+            error_names[i] = error_str
+
     # Write the data.
     write_spin_data(file, mol_names=mol_names, res_nums=res_nums, res_names=res_names, spin_nums=spin_nums, spin_names=spin_names, data=values, data_name=data_names, error=errors, error_name=error_names)
 
