@@ -33,12 +33,17 @@
 # Module docstring.
 """The main module for relax execution."""
 
-# Set up the Python 2 and 3 work-arounds.
-import lib.compat
-
 # Dependency checks.
 import dep_check
 
+# Eliminate the ^[[?1034h escape code being produced on Linux systems by the import of the readline module.
+import os
+if 'TERM' in os.environ and os.environ['TERM'] == 'xterm':
+    os.environ['TERM'] = 'linux'
+
+# Set up the Python 2 and 3 work-arounds.
+import lib.compat
+
 # Python modules.
 import numpy
 from optparse import Option, OptionGroup, OptionParser
@@ -307,11 +312,12 @@ class Relax:
         # Recognised command line options for the test suite.
         group = OptionGroup(parser, 'Test suite options')
         group.add_option('-x', '--test-suite', action='store_true', dest='test_suite', default=0, help='execute the full relax test suite')
-        group.add_option('-s', '--system-tests', action='store_true', dest='system_tests', default=0, help='execute the system/functional tests')
-        group.add_option('-u', '--unit-tests', action='store_true', dest='unit_tests', default=0, help='execute the unit tests')
-        group.add_option('--gui-tests', action='store_true', dest='gui_tests', default=0, help='execute the GUI tests')
-        group.add_option('--verification-tests', action='store_true', dest='verification_tests', default=0, help='execute the software verification tests')
-        group.add_option('--time', action='store_true', dest='tt', default=0, help='enable the timing of individual tests in the test suite')
+        group.add_option('-s', '--system-tests', action='store_true', dest='system_tests', default=0, help='execute the system/functional tests.  Test names, revealed with the --time option, can be supplied to perform a subset of all tests.')
+        group.add_option('-u', '--unit-tests', action='store_true', dest='unit_tests', default=0, help='execute the unit tests.  Module names, revealed with the --time option, can be supplied to perform a subset of all tests.')
+        group.add_option('--gui-tests', action='store_true', dest='gui_tests', default=0, help='execute the GUI tests.  Test names, revealed with the --time option, can be supplied to perform a subset of all tests.')
+        group.add_option('--verification-tests', action='store_true', dest='verification_tests', default=0, help='execute the software verification tests.  Test names, revealed with the --time option, can be supplied to perform a subset of all tests.')
+        group.add_option('--time', action='store_true', dest='tt', default=0, help='print out the timings of individual tests in the test suite')
+        group.add_option('--no-skip', action='store_true', dest='no_skip', default=0, help='a debugging option for relax developers to turn on all blacklisted tests, even those that will fail')
         parser.add_option_group(group)
 
         # Recognised command line options for debugging.
@@ -383,6 +389,11 @@ class Relax:
             if options.tt:
                 self.test_timings = True
 
+            # Run blacklisted tests.
+            status.skip_blacklisted_tests = True
+            if options.no_skip:
+                status.skip_blacklisted_tests = False
+
         # The argument is a script.
         else:
             # Number of positional arguments should only be 0 or 1.  1 should be the script file.
