@@ -36,11 +36,12 @@ from lib.geometry.rotations import euler_to_R_zyz, two_vect_to_R
 from lib.io import open_write_file
 from lib.order import order_parameters
 from lib.structure.cones import Iso_cone, Pseudo_elliptic
+from lib.structure.geometric import generate_vector_residues
 from lib.structure.internal.object import Internal
+from lib.structure.represent.cone import cone
 from lib.structure.represent.rotor import rotor_pdb
 from lib.text.sectioning import subsection
-from pipe_control import pipes
-from pipe_control.structure import geometric
+from pipe_control.pipes import check_pipe
 from pipe_control.structure.mass import pipe_centre_of_mass
 from specific_analyses.frame_order.data import domain_moving, translation_fixed
 from specific_analyses.frame_order.parameters import update_model
@@ -56,7 +57,7 @@ def average_position(pivot='com', translation=True):
     """
 
     # Test if the current data pipe exists.
-    pipes.test()
+    check_pipe()
 
     # Check the pivot value.
     if pivot not in ['com', 'motional']:
@@ -75,7 +76,7 @@ def num_int_pts(num=200000):
     """
 
     # Test if the current data pipe exists.
-    pipes.test()
+    check_pipe()
 
     # Store the value.
     cdp.num_int_pts = num
@@ -98,6 +99,9 @@ def pdb_ave_pos(file=None, dir=None, force=False):
     # Make a copy of the structural object (so as to preserve the original structure).
     structure = deepcopy(cdp.structure)
 
+    # The internal structural selection object.
+    selection = structure.selection(atom_id=domain_moving())
+
     # First rotate the moving domain to the average position.
     R = zeros((3, 3), float64)
     if hasattr(cdp, 'ave_pos_alpha'):
@@ -108,11 +112,11 @@ def pdb_ave_pos(file=None, dir=None, force=False):
         origin = pipe_centre_of_mass(atom_id=domain_moving(), verbosity=0)
     else:
         origin = array([cdp.pivot_x, cdp.pivot_y, cdp.pivot_z])
-    structure.rotate(R=R, origin=origin, atom_id=domain_moving())
+    structure.rotate(R=R, origin=origin, selection=selection)
 
     # Then translate the moving domain.
     if not translation_fixed():
-        structure.translate(T=[cdp.ave_pos_x, cdp.ave_pos_y, cdp.ave_pos_z], atom_id=domain_moving())
+        structure.translate(T=[cdp.ave_pos_x, cdp.ave_pos_y, cdp.ave_pos_z], selection=selection)
 
     # Write out the PDB file.
     file = open_write_file(file_name=file, dir=dir, force=force)
@@ -148,7 +152,7 @@ def pdb_geometric_rep(file=None, dir=None, size=30.0, inc=36, force=False, neg_c
     @type inc:              int
     @keyword force:         Flag which if set to True will cause any pre-existing file to be overwritten.
     @type force:            bool
-    @keyword neg_cone:      A flag which if True will cause the negative cone to be added to the representation.
+    @keyword neg_cone:      A flag which if True will cause the negative cone to be added to the representation.  This is ignored for the rotor models.
     @type neg_cone:         bool
     """
 
@@ -169,9 +173,13 @@ def pdb_geometric_rep(file=None, dir=None, size=30.0, inc=36, force=False, neg_c
     structure = Internal()
 
     # Create model for the positive and negative images.
+    if cdp.model in ['rotor', 'free rotor']:
+        neg_cone = False
     model = structure.add_model(model=1)
+    model_num = 1
     if neg_cone:
         model_neg = structure.add_model(model=2)
+        model_num = 2
 
     # The pivot point.
     pivot = array([cdp.pivot_x, cdp.pivot_y, cdp.pivot_z])
@@ -185,7 +193,9 @@ def pdb_geometric_rep(file=None, dir=None, size=30.0, inc=36, force=False, neg_c
             rotor_angle = cdp.cone_sigma_max
 
         # Get the CoM of the entire molecule to use as the centre of the rotor.
-        com = pipe_centre_of_mass(verbosity=0)
+        com = pivot
+        if cdp.model in ['rotor', 'free rotor']:
+            com = pipe_centre_of_mass(verbosity=0)
 
         # Generate the rotor axis.
         if cdp.model in ['rotor']:
@@ -195,15 +205,14 @@ def pdb_geometric_rep(file=None, dir=None, size=30.0, inc=36, force=False, neg_c
         else:
             axis = create_rotor_axis_euler(alpha=cdp.eigen_alpha, beta=cdp.eigen_beta, gamma=cdp.eigen_gamma)
 
-        # Add the rotor object to the structure as a new molecule.
-        rotor_pdb(structure=structure, rotor_angle=rotor_angle, axis=axis, axis_pt=pivot, centre=com, span=2e-9, blade_length=5e-10, staggered=False)
+        # The size of the rotor, taking the 30 Angstrom cone representation into account.
+        if cdp.model in ['rotor', 'free rotor']:
+            span = 20e-10
+        else:
+            span = 35e-10
 
-    # FIXME:  Temporary write out and exit.
-    print("\nGenerating the PDB file.")
-    pdb_file = open_write_file(file, dir, force=force)
-    structure.write_pdb(pdb_file)
-    pdb_file.close()
-    return
+        # Add the rotor object to the structure as a new molecule.
+        rotor_pdb(structure=structure, rotor_angle=rotor_angle, axis=axis, axis_pt=pivot, centre=com, span=span, blade_length=5e-10, staggered=True)
 
     # Create the molecule.
     structure.add_molecule(name='rest')
@@ -230,7 +239,10 @@ def pdb_geometric_rep(file=None, dir=None, size=30.0, inc=36, force=False, neg_c
         print("\nGenerating the z-axis system.")
 
         # The axis.
-        axis = create_rotor_axis_spherical(theta=cdp.axis_theta, phi=cdp.axis_phi)
+        if cdp.model in ['rotor']:
+            axis = create_rotor_axis_alpha(alpha=cdp.axis_alpha, pivot=pivot, point=com)
+        else:
+            axis = create_rotor_axis_spherical(theta=cdp.axis_theta, phi=cdp.axis_phi)
         print(("Central axis: %s." % axis))
 
         # Rotations and inversions.
@@ -246,7 +258,10 @@ def pdb_geometric_rep(file=None, dir=None, size=30.0, inc=36, force=False, neg_c
 
             # Fill the structure.
             for i in range(cdp.sim_number):
-                axis_sim[i] = create_rotor_axis_spherical(theta=cdp.axis_theta_sim[i], phi=cdp.axis_phi_sim[i])
+                if cdp.model in ['rotor']:
+                    axis_sim[i] = create_rotor_axis_alpha(alpha=cdp.axis_alpha_sim[i], pivot=pivot, point=com)
+                else:
+                    axis_sim[i] = create_rotor_axis_spherical(theta=cdp.axis_theta_sim[i], phi=cdp.axis_phi_sim[i])
 
             # Inversion.
             axis_sim_pos = axis_sim
@@ -254,11 +269,11 @@ def pdb_geometric_rep(file=None, dir=None, size=30.0, inc=36, force=False, neg_c
 
         # Generate the axis vectors.
         print("\nGenerating the axis vectors.")
-        res_num = geometric.generate_vector_residues(mol=mol, vector=axis_pos, atom_name='z-ax', res_name_vect='AXE', sim_vectors=axis_sim_pos, res_num=2, origin=pivot, scale=size)
+        res_num = generate_vector_residues(mol=mol, vector=axis_pos, atom_name='z-ax', res_name_vect='AXE', sim_vectors=axis_sim_pos, res_num=2, origin=pivot, scale=size)
 
         # The negative.
         if neg_cone:
-            res_num = geometric.generate_vector_residues(mol=mol_neg, vector=axis_neg, atom_name='z-ax', res_name_vect='AXE', sim_vectors=axis_sim_neg, res_num=2, origin=pivot, scale=size)
+            res_num = generate_vector_residues(mol=mol_neg, vector=axis_neg, atom_name='z-ax', res_name_vect='AXE', sim_vectors=axis_sim_neg, res_num=2, origin=pivot, scale=size)
 
     # The full axis system.
     else:
@@ -303,9 +318,9 @@ def pdb_geometric_rep(file=None, dir=None, size=30.0, inc=36, force=False, neg_c
                 axis_sim_neg = axes_sim_neg[:,:, j]
 
             # The vectors.
-            res_num = geometric.generate_vector_residues(mol=mol, vector=axes_pos[:, j], atom_name='%s-ax'%label[j], res_name_vect='AXE', sim_vectors=axis_sim_pos, res_num=2, origin=pivot, scale=size)
+            res_num = generate_vector_residues(mol=mol, vector=axes_pos[:, j], atom_name='%s-ax'%label[j], res_name_vect='AXE', sim_vectors=axis_sim_pos, res_num=2, origin=pivot, scale=size)
             if neg_cone:
-                res_num = geometric.generate_vector_residues(mol=mol_neg, vector=axes_neg[:, j], atom_name='%s-ax'%label[j], res_name_vect='AXE', sim_vectors=axis_sim_neg, res_num=2, origin=pivot, scale=size)
+                res_num = generate_vector_residues(mol=mol_neg, vector=axes_neg[:, j], atom_name='%s-ax'%label[j], res_name_vect='AXE', sim_vectors=axis_sim_neg, res_num=2, origin=pivot, scale=size)
 
 
     # The cone object.
@@ -326,7 +341,7 @@ def pdb_geometric_rep(file=None, dir=None, size=30.0, inc=36, force=False, neg_c
 
         # The pseudo-ellipse cone object.
         if cdp.model in ['pseudo-ellipse', 'pseudo-ellipse, torsionless', 'pseudo-ellipse, free rotor']:
-            cone = Pseudo_elliptic(cdp.cone_theta_x, cdp.cone_theta_y)
+            cone_obj = Pseudo_elliptic(cdp.cone_theta_x, cdp.cone_theta_y)
 
         # The isotropic cone object.
         else:
@@ -337,14 +352,14 @@ def pdb_geometric_rep(file=None, dir=None, size=30.0, inc=36, force=False, neg_c
                 cone_theta = order_parameters.iso_cone_S_to_theta(cdp.cone_s1)
 
             # The object.
-            cone = Iso_cone(cone_theta)
+            cone_obj = Iso_cone(cone_theta)
 
         # Create the positive and negative cones.
-        geometric.create_cone_pdb(mol=mol, cone=cone, start_res=mol.res_num[-1]+1, apex=pivot, R=R_pos, inc=inc, distribution='regular', axis_flag=False)
+        cone(mol=mol, cone_obj=cone_obj, start_res=mol.res_num[-1]+1, apex=pivot, R=R_pos, inc=inc, distribution='regular', axis_flag=False)
 
         # The negative.
         if neg_cone:
-            geometric.create_cone_pdb(mol=mol_neg, cone=cone, start_res=mol_neg.res_num[-1]+1, apex=pivot, R=R_neg, inc=inc, distribution='regular', axis_flag=False)
+            cone(mol=mol_neg, cone_obj=cone_obj, start_res=mol_neg.res_num[-1]+1, apex=pivot, R=R_neg, inc=inc, distribution='regular', axis_flag=False)
 
 
     # Create the PDB file.
@@ -385,7 +400,7 @@ def pdb_model(ave_pos_file="ave_pos.pdb", rep_file="frame_order.pdb", dist_file=
         raise RelaxError("Minimally one PDB file name must be supplied.")
 
     # Test if the current data pipe exists.
-    pipes.test()
+    check_pipe()
 
     # Create the average position structure.
     if ave_pos_file:
@@ -416,7 +431,7 @@ def pivot(pivot=None, order=1, fix=False):
     """
 
     # Test if the current data pipe exists.
-    pipes.test()
+    check_pipe()
 
     # Store the fixed flag.
     cdp.pivot_fixed = fix
@@ -462,7 +477,7 @@ def quad_int(flag=False):
     """
 
     # Test if the current data pipe exists.
-    pipes.test()
+    check_pipe()
 
     # Store the flag.
     cdp.quad_int = flag
@@ -476,7 +491,7 @@ def ref_domain(ref=None):
     """
 
     # Test if the current data pipe exists.
-    pipes.test()
+    check_pipe()
 
     # Check that the domain is defined.
     if not hasattr(cdp, 'domain') or ref not in list(cdp.domain.keys()):
@@ -506,7 +521,7 @@ def select_model(model=None):
     """
 
     # Test if the current data pipe exists.
-    pipes.test()
+    check_pipe()
 
     # Test if the model name exists.
     if not model in ['pseudo-ellipse', 'pseudo-ellipse, torsionless', 'pseudo-ellipse, free rotor', 'iso cone', 'iso cone, torsionless', 'iso cone, free rotor', 'line', 'line, torsionless', 'line, free rotor', 'rotor', 'rigid', 'free rotor', 'double rotor']:
