@@ -2,6 +2,7 @@
 #                                                                             #
 # Copyright (C) 2004-2014 Edward d'Auvergne                                   #
 # Copyright (C) 2009 Sebastien Morin                                          #
+# Copyright (C) 2014 Troels E. Linnet                                         #
 #                                                                             #
 # This file is part of the program relax (http://www.nmr-relax.com).          #
 #                                                                             #
@@ -25,16 +26,17 @@
 
 # Python module imports.
 from copy import deepcopy
-from numpy import array, float64, identity, median, zeros
+from numpy import array, float64, median, zeros
 import sys
 
 # relax module imports.
+from lib.dispersion.variables import MODEL_LIST_MMQ, MODEL_M61B, MODEL_NS_MMQ_3SITE, MODEL_NS_MMQ_3SITE_LINEAR, MODEL_NS_R1RHO_3SITE, MODEL_NS_R1RHO_3SITE_LINEAR, PARAMS_R20
 from lib.errors import RelaxError, RelaxNoSequenceError
 from lib.text.sectioning import subsection
 from pipe_control import pipes
-from pipe_control.mol_res_spin import exists_mol_res_spin_data, return_spin
-from specific_analyses.relax_disp.data import count_spins, generate_r20_key, has_exponential_exp_type, loop_cluster, loop_exp_frq
-from specific_analyses.relax_disp.variables import MODEL_LIST_MMQ, MODEL_M61B, MODEL_NS_MMQ_3SITE, MODEL_NS_MMQ_3SITE_LINEAR, MODEL_NS_R1RHO_3SITE, MODEL_NS_R1RHO_3SITE_LINEAR, PARAMS_R20
+from pipe_control.mol_res_spin import exists_mol_res_spin_data, return_spin, spin_loop
+from pipe_control.pipes import check_pipe
+from specific_analyses.relax_disp.data import count_spins, generate_r20_key, has_exponential_exp_type, is_r1_optimised, loop_cluster, loop_exp_frq
 
 
 def assemble_param_vector(spins=None, key=None, sim_index=None):
@@ -70,58 +72,6 @@ def assemble_param_vector(spins=None, key=None, sim_index=None):
     return array(param_vector, float64)
 
 
-def assemble_scaling_matrix(spins=None, key=None, scaling=True):
-    """Create and return the scaling matrix.
-
-    @keyword spins:         The list of spin data containers for the block.
-    @type spins:            list of SpinContainer instances
-    @keyword key:           The key for the R2eff and I0 parameters.
-    @type key:              str or None
-    @keyword scaling:       A flag which if False will cause the identity matrix to be returned.
-    @type scaling:          bool
-    @return:                The diagonal and square scaling matrix.
-    @rtype:                 numpy diagonal matrix
-    """
-
-    # Initialise.
-    scaling_matrix = identity(param_num(spins=spins), float64)
-    i = 0
-    param_index = 0
-
-    # No diagonal scaling.
-    if not scaling:
-        return scaling_matrix
-
-    # Loop over the parameters of the cluster.
-    for param_name, param_index, spin_index, r20_key in loop_parameters(spins=spins):
-        # Transversal relaxation rate scaling.
-        if param_name in ['r2', 'r2a', 'r2b']:
-            scaling_matrix[param_index, param_index] = 10
-
-        # The pA.pB.dw**2, phi_ex_B, phi_ex_C and pA.dw**2 parameters.
-        elif param_name in ['phi_ex', 'phi_ex_B', 'phi_ex_C', 'padw2']:
-            scaling_matrix[param_index, param_index] = 1
-
-        # Chemical shift difference between states A and B scaling.
-        elif param_name in ['dw', 'dw_AB', 'dw_AC', 'dw_BC', 'dwH', 'dwH_AB', 'dwH_AC', 'dwH_BC']:
-            scaling_matrix[param_index, param_index] = 1
-
-        # The population of state X.
-        elif param_name in ['pA', 'pB', 'pC']:
-            scaling_matrix[param_index, param_index] = 1
-
-        # Exchange rate scaling.
-        elif param_name in ['kex', 'kex_AB', 'kex_AC', 'kex_BC', 'k_AB', 'kB', 'kC']:
-            scaling_matrix[param_index, param_index] = 10000
-
-        # Time of exchange scaling.
-        elif param_name == 'tex':
-            scaling_matrix[param_index, param_index] = 1e-4
-
-    # Return the scaling matrix.
-    return scaling_matrix
-
-
 def copy(pipe_from=None, pipe_to=None):
     """Copy dispersion parameters from one data pipe to another, taking the median of previous values to a start value for clusters.
     Taking the median prevent averaging extreme outliers.
@@ -140,8 +90,8 @@ def copy(pipe_from=None, pipe_to=None):
         pipe_to = pipe_orig
 
     # Test that the pipes exist.
-    pipes.test(pipe_from)
-    pipes.test(pipe_to)
+    check_pipe(pipe_from)
+    check_pipe(pipe_to)
 
     # Test that the pipes are not the same.
     if pipe_from == pipe_to:
@@ -318,7 +268,7 @@ def copy(pipe_from=None, pipe_to=None):
 
             # All other spin specific parameters.
             for param in spin_from.params:
-                if param in ['r2', 'pA', 'pB', 'pC', 'kex', 'kex_AB', 'kex_AC', 'kex_BC', 'k_AB', 'kB', 'kC', 'tex']:
+                if param in ['r2', 'r2a', 'r2b', 'pA', 'pB', 'pC', 'kex', 'kex_AB', 'kex_AC', 'kex_BC', 'k_AB', 'kB', 'kC', 'tex']:
                     continue
 
                 # Copy the value.
@@ -373,11 +323,13 @@ def disassemble_param_vector(param_vector=None, key=None, spins=None, sim_index=
         set_value(value=param_vector[param_index], key=key, spins=spins, sim_index=sim_index, param_name=param_name, spin_index=spin_index, r20_key=r20_key)
 
 
-def get_param_names(spins=None):
+def get_param_names(spins=None, full=False):
     """Generate a list of dispersion parameter names for the given spins.
 
-    @keyword spins:         The list of spin data containers for the block.
-    @type spins:            list of SpinContainer instances
+    @keyword spins: The list of spin data containers for the block.
+    @type spins:    list of SpinContainer instances
+    @keyword full:  A flag which if True will add the R2 key to the parameter names.
+    @type full:     bool
     """
 
     # Initialise the structure.
@@ -389,7 +341,7 @@ def get_param_names(spins=None):
         param_text = param_name
 
         # The parameters with additional details.
-        if param_name in ['r2', 'r2a', 'r2b']:
+        if full and param_name in PARAMS_R20:
             param_text += " (%s)" % r20_key
  
         # Append the text.
@@ -418,8 +370,8 @@ def get_value(key=None, spins=None, sim_index=None, param_name=None, spin_index=
     @rtype:                 float
     """
 
-    # Default value of 0.0.
-    value = 0.0
+    # Default value of None.
+    value = None
 
     # Spin specific parameters.
     if spin_index != None:
@@ -480,6 +432,7 @@ def linear_constraints(spins=None, scaling_matrix=None):
 
     The different constraints used within different models are::
 
+        0 <= R1 <= 200
         0 <= R2 <= 200
         0 <= R2A <= 200
         0 <= R2B <= 200
@@ -493,11 +446,10 @@ def linear_constraints(spins=None, scaling_matrix=None):
         padw2 >= 0
         dw >= 0
         0 <= kex <= 2e6
-        0 <= k_AB <= 2e6
         0 <= kB <= 2e6
         0 <= kC <= 2e6
+        0 <= k_AB <= 100
         tex >= 0
-        k_AB >= 0
 
 
     Matrix notation
@@ -505,6 +457,10 @@ def linear_constraints(spins=None, scaling_matrix=None):
 
     In the notation A.x >= b, where A is a matrix of coefficients, x is an array of parameter values, and b is a vector of scalars, these inequality constraints are::
 
+        | 1  0  0 |     |    R1    |      |    0    |
+        |         |     |          |      |         |
+        |-1  0  0 |     |    R1    |      |  -200   |
+        |         |     |          |      |         |
         | 1  0  0 |     |    R2    |      |    0    |
         |         |     |          |      |         |
         |-1  0  0 |     |    R2    |      |  -200   |
@@ -547,9 +503,11 @@ def linear_constraints(spins=None, scaling_matrix=None):
         |         |     |          |      |         |
         |-1  0  0 |     |    kC    |      |  -2e6   |
         |         |     |          |      |         |
-        | 1  0  0 |     |   tex    |      |    0    |
-        |         |     |          |      |         |
         | 1  0  0 |     |   k_AB   |      |    0    |
+        |         |     |          |      |         |
+        |-1  0  0 |     |   k_AB   |      |   100   |
+        |         |     |          |      |         |
+        | 1  0  0 |     |   tex    |      |    0    |
 
 
     @keyword spins:             The list of spin data containers for the block.
@@ -586,8 +544,18 @@ def linear_constraints(spins=None, scaling_matrix=None):
             b.append(0.0)
             j += 1
 
+        # The longitudinal relaxation rates (0 <= r1 <= 200).
+        elif param_name in ['r1']:
+            A.append(zero_array * 0.0)
+            A.append(zero_array * 0.0)
+            A[j][param_index] = 1.0
+            A[j+1][param_index] = -1.0
+            b.append(0.0)
+            b.append(-200.0 / scaling_matrix[param_index, param_index])
+            j += 2
+
         # The transversal relaxation rates (0 <= r2 <= 200).
-        elif param_name in ['r2', 'r2a', 'r2b']:
+        elif param_name in PARAMS_R20:
             A.append(zero_array * 0.0)
             A.append(zero_array * 0.0)
             A[j][param_index] = 1.0
@@ -654,7 +622,7 @@ def linear_constraints(spins=None, scaling_matrix=None):
                     break
 
         # Exchange rates and times (0 <= k <= 2e6).
-        elif param_name in ['kex', 'kex_AB', 'kex_AC', 'kex_BC', 'k_AB', 'kB', 'kC']:
+        elif param_name in ['kex', 'kex_AB', 'kex_AC', 'kex_BC', 'kB', 'kC']:
             A.append(zero_array * 0.0)
             A.append(zero_array * 0.0)
             A[j][param_index] = 1.0
@@ -663,6 +631,16 @@ def linear_constraints(spins=None, scaling_matrix=None):
             b.append(-2e6 / scaling_matrix[param_index, param_index])
             j += 2
 
+        # Exchange rates and times (0 <= k_AB <= 100).
+        elif param_name in ['k_AB']:
+            A.append(zero_array * 0.0)
+            A.append(zero_array * 0.0)
+            A[j][param_index] = 1.0
+            A[j+1][param_index] = -1.0
+            b.append(0.0)
+            b.append(-100. / scaling_matrix[param_index, param_index])
+            j += 2
+
         # Exchange times (tex >= 0).
         elif param_name in ['tex']:
             A.append(zero_array * 0.0)
@@ -687,6 +665,9 @@ def loop_parameters(spins=None):
     @rtype:         str, int, int, str
     """
 
+    # Make sure that the R1 parameter is correctly set up.
+    r1_setup()
+
     # The parameter index.
     param_index = -1
 
@@ -709,26 +690,38 @@ def loop_parameters(spins=None):
 
     # All other models.
     else:
-        # First the R2 parameters (one per spin per field strength).
+        # First the R1 fit parameter (one per spin per field strength).
+        for spin_index in range(len(spins)):
+            # Skip deselected spins.
+            if not spins[spin_index].select:
+                continue
+
+            # The R1 parameter.
+            if 'r1' in spins[spin_index].params:
+                for exp_type, frq in loop_exp_frq():
+                    param_index += 1
+                    yield 'r1', param_index, spin_index, generate_r20_key(exp_type=exp_type, frq=frq)
+
+        # Then the R2 parameters (one per spin per field strength).
         for spin_index in range(len(spins)):
             # Skip deselected spins.
             if not spins[spin_index].select:
                 continue
 
             # The R2 parameter.
-            if 'r2' in spins[0].params:
+            if 'r2' in spins[spin_index].params:
                 for exp_type, frq in loop_exp_frq():
                     param_index += 1
                     yield 'r2', param_index, spin_index, generate_r20_key(exp_type=exp_type, frq=frq)
 
             # The R2A parameter.
-            if 'r2a' in spins[0].params:
+            if 'r2a' in spins[spin_index].params:
                 for exp_type, frq in loop_exp_frq():
                     param_index += 1
                     yield 'r2a', param_index, spin_index, generate_r20_key(exp_type=exp_type, frq=frq)
 
             # The R2B parameter.
-            if 'r2b' in spins[0].params:
+            if 'r2b' in spins[spin_index].params:
                 for exp_type, frq in loop_exp_frq():
                     param_index += 1
                     yield 'r2b', param_index, spin_index, generate_r20_key(exp_type=exp_type, frq=frq)
@@ -786,7 +779,7 @@ def loop_parameters(spins=None):
 
         # All other parameters (one per spin cluster).
         for param in spins[0].params:
-            if not param in ['r2', 'r2a', 'r2b', 'phi_ex', 'phi_ex_B', 'phi_ex_C', 'padw2', 'dw', 'dw_AB', 'dw_BC', 'dw_AB', 'dwH', 'dwH_AB', 'dwH_BC', 'dwH_AB']:
+            if not param in ['r1', 'r2', 'r2a', 'r2b', 'phi_ex', 'phi_ex_B', 'phi_ex_C', 'padw2', 'dw', 'dw_AB', 'dw_BC', 'dw_AB', 'dwH', 'dwH_AB', 'dwH_BC', 'dwH_AB']:
                 param_index += 1
                 yield param, param_index, None, None
 
@@ -899,6 +892,17 @@ def param_num(spins=None):
         if len(spin.params) != len(spins[0].params):
             raise RelaxError("The number of parameters for each spin in the cluster are not the same.")
 
+    # Count the number of R10 parameters.
+    for spin in spins:
+        # Skip deselected spins.
+        if not spin.select:
+            continue
+
+        for i in range(len(spin.params)):
+            if spin.params[i] in ['r1']:
+                for exp_type, frq in loop_exp_frq():
+                    num += 1
+
     # Count the number of R20 parameters.
     for spin in spins:
         # Skip deselected spins.
@@ -922,7 +926,7 @@ def param_num(spins=None):
                 num += 1
 
     # Count all other parameters, but only for a single spin.
-    all_params = PARAMS_R20 + spin_params
+    all_params = ['r1'] + PARAMS_R20 + spin_params
     for spin in spins:
         # Skip deselected spins.
         if not spin.select:
@@ -937,6 +941,29 @@ def param_num(spins=None):
     return num
 
 
+def r1_setup():
+    """Modify the current spin parameter list to either include or exclude the R1 parameter."""
+
+    # Loop over the spins.
+    for spin, spin_id in spin_loop(return_id=True, skip_desel=True):
+        # No model set up.
+        if not hasattr(spin, 'params') or not hasattr(spin, 'model'):
+            continue
+
+        # Should R1 data be optimised?
+        r1_fit = is_r1_optimised(spin.model)
+
+        # Prepend R1 and add it to the spin container.
+        if r1_fit and 'r1' not in spin.params:
+            spin.params.insert(0, 'r1')
+        if r1_fit and not hasattr(spin, 'r1'):
+            spin.r1 = {}
+
+        # Remove the R1 parameter.
+        if not r1_fit and 'r1' in spin.params:
+            spin.params.remove('r1')
+
+
 def set_value(value=None, key=None, spins=None, sim_index=None, param_name=None, spin_index=None, r20_key=None):
     """Return the value for the given parameter.
 
