@@ -34,10 +34,10 @@ import dep_check
 from gui.interpreter import Interpreter; interpreter = Interpreter()
 from gui.string_conv import float_to_gui, str_to_gui
 from gui.uf_objects import Uf_storage; uf_store = Uf_storage()
+from lib.dispersion.variables import EXP_TYPE_R1RHO, MODEL_CR72, MODEL_DPL94, MODEL_IT99, MODEL_LM63, MODEL_NOREX, MODEL_NS_CPMG_2SITE_EXPANDED, MODEL_R2EFF, MODEL_TP02
 from pipe_control.mol_res_spin import spin_loop
 from pipe_control.pipes import switch
 from specific_analyses.relax_disp.data import generate_r20_key
-from specific_analyses.relax_disp.variables import EXP_TYPE_R1RHO, MODEL_CR72, MODEL_IT99, MODEL_LM63, MODEL_NOREX, MODEL_NS_CPMG_2SITE_EXPANDED, MODEL_R2EFF, MODEL_TP02
 from status import Status; status = Status()
 from test_suite.gui_tests.base_classes import GuiTestCase
 
@@ -207,6 +207,29 @@ class Relax_disp(GuiTestCase):
         wizard.Close()
 
 
+    def test_bug_22501_close_all_analyses(self):
+        """Test catching U{bug #22501<https://gna.org/bugs/index.php?22501>}, 'Close all analyses' raises error."""
+
+        # Simulate the new analysis wizard, selecting the fixed time CPMG experiment.
+        self.app.gui.analysis.menu_new(None)
+        page = self.app.gui.analysis.new_wizard.wizard.get_page(0)
+        page.select_disp(None)
+        self.app.gui.analysis.new_wizard.wizard._go_next(None)
+        self.app.gui.analysis.new_wizard.wizard._go_next(None)
+
+        # Get the data.
+        analysis_type, analysis_name, pipe_name, pipe_bundle, uf_exec = self.app.gui.analysis.new_wizard.get_data()
+
+        # Set up the analysis.
+        self.app.gui.analysis.new_analysis(analysis_type=analysis_type, analysis_name=analysis_name, pipe_name=pipe_name, pipe_bundle=pipe_bundle, uf_exec=uf_exec)
+
+        # Alias the analysis.
+        analysis = self.app.gui.analysis.get_page_from_name("Relaxation dispersion")
+
+        # Closure.
+        self.app.gui.analysis.delete_analysis(0)
+
+
     def test_hansen_trunc_data(self):
         """Test the GUI analysis with Flemming Hansen's CPMG data truncated to residues 70 and 71."""
 
@@ -520,6 +543,69 @@ class Relax_disp(GuiTestCase):
         self.assertEqual(cdp.mol[0].res[2].spin[0].name, 'N')
 
 
+    def test_r2eff_err_estimate(self):
+        """Test U{task #7822:<https://gna.org/task/?7822>}, Implement user function to estimate R2eff and associated errors for exponential curve fitting.."""
+
+        # The paths to the data files.
+        data_path = status.install_path + sep+'test_suite'+sep+'shared_data'+sep+'dispersion'+sep+'Kjaergaard_et_al_2013'+sep
+
+        # Simulate the new analysis wizard, selecting the fixed time CPMG experiment.
+        self.app.gui.analysis.menu_new(None)
+        page = self.app.gui.analysis.new_wizard.wizard.get_page(0)
+        page.select_disp(None)
+        self.app.gui.analysis.new_wizard.wizard._go_next(None)
+        self.app.gui.analysis.new_wizard.wizard._go_next(None)
+
+        # Get the data.
+        analysis_type, analysis_name, pipe_name, pipe_bundle, uf_exec = self.app.gui.analysis.new_wizard.get_data()
+
+        # Set up the analysis.
+        self.app.gui.analysis.new_analysis(analysis_type=analysis_type, analysis_name=analysis_name, pipe_name=pipe_name, pipe_bundle=pipe_bundle, uf_exec=uf_exec)
+
+        # Alias the analysis.
+        analysis = self.app.gui.analysis.get_page_from_name("Relaxation dispersion")
+
+        # Change the results directory.
+        analysis.field_results_dir.SetValue(str_to_gui(ds.tmpdir))
+
+        # Load the sequence.
+        file = data_path + '1_setup_r1rho_GUI.py'
+        self._execute_uf(uf_name='script', file=file, dir=None)
+
+        # De select spins
+        self._execute_uf(uf_name='deselect.spin', spin_id=":1-100")
+        self._execute_uf(uf_name='select.spin', spin_id=":52@N")
+
+        # Deselect all but the few models.
+        models = [MODEL_R2EFF, MODEL_NOREX, MODEL_DPL94]
+        for i in range(len(analysis.model_field.models_stripped)):
+            if analysis.model_field.models_stripped[i] in models:
+                analysis.model_field.select[i] = True
+            else:
+                analysis.model_field.select[i] = False
+        analysis.model_field.modify()
+
+        # Set the grid search size and number of MC sims.
+        analysis.grid_inc.SetValue(0)
+        analysis.mc_sim_num.SetValue(3)
+        analysis.exp_mc_sim_num.SetValue(-1)
+
+        # Do fitting of R1.
+        analysis.r1_fit.SetValue(bool(True))
+
+        # Execute relax.
+        analysis.execute(wx.CommandEvent(wx.wxEVT_COMMAND_BUTTON_CLICKED, analysis.button_exec_relax.GetId()))
+
+        # Wait for execution to complete.
+        analysis.thread.join()
+
+        # Flush all wx events.
+        wx.Yield()
+
+        # Exceptions in the thread.
+        self.check_exceptions()
+
+
     def test_tp02_data_to_tp02(self):
         """Test the GUI analysis with the relaxation dispersion 'TP02' model fitting to the 'TP02' synthetic data."""
 
