@@ -226,7 +226,7 @@ class Frame_order(SystemTestCase):
         self.interpreter.intro_off()
 
         # Check the minimum.
-        self.interpreter.calc()
+        self.interpreter.minimise.calculate()
         print("%-20s %10.5f" % ("chi2 minimum", cdp.chi2))
         self.assertAlmostEqual(cdp.chi2, ref_chi2)
 
@@ -240,13 +240,13 @@ class Frame_order(SystemTestCase):
 
             # Deviate upwards.
             setattr(cdp, param, curr+delta)
-            self.interpreter.calc()
+            self.interpreter.minimise.calculate()
             print("%-20s %10.5f" % ("chi2 up", cdp.chi2))
             self.assert_(cdp.chi2 > ref_chi2)
 
             # Deviate downwards.
             setattr(cdp, param, curr-delta)
-            self.interpreter.calc()
+            self.interpreter.minimise.calculate()
             print("%-20s %10.5f" % ("chi2 down", cdp.chi2))
             self.assert_(cdp.chi2 > ref_chi2)
 
@@ -348,7 +348,7 @@ class Frame_order(SystemTestCase):
         """Test the isotropic cone, free rotor frame order model of CaM."""
 
         # The flags, execute the script, and then check the chi2 value.
-        self.flags()
+        self.flags(opt=True)
         self.interpreter.run(script_file=self.cam_path+'iso_cone.py')
         self.check_chi2(4.4891048094666539)
 
