@@ -1,6 +1,7 @@
 ###############################################################################
 #                                                                             #
 # Copyright (C) 2013-2014 Troels E. Linnet                                    #
+# Copyright (C) 2014 Edward d'Auvergne                                        #
 #                                                                             #
 # This file is part of the program relax (http://www.nmr-relax.com).          #
 #                                                                             #
@@ -21,29 +22,25 @@
 # Script for calculating synthetics CPMG data.
 
 # Python module imports.
-from os import sep
-from tempfile import mkdtemp
 from math import sqrt
-import sys
 
 # relax module imports.
-from auto_analyses.relax_disp import Relax_disp
 from lib.io import open_write_file
 from data_store import Relax_data_store; ds = Relax_data_store()
-from pipe_control.mol_res_spin import return_spin, spin_loop
+from pipe_control.mol_res_spin import return_spin
 from specific_analyses.relax_disp.data import generate_r20_key, loop_exp_frq, loop_offset_point
 from specific_analyses.relax_disp import optimisation
 from status import Status; status = Status()
 # The variables already defined in relax.
-from specific_analyses.relax_disp.variables import EXP_TYPE_CPMG_SQ, MODEL_PARAMS
+from lib.dispersion.variables import EXP_TYPE_CPMG_SQ, MODEL_PARAMS
 # Analytical
-from specific_analyses.relax_disp.variables import MODEL_CR72, MODEL_IT99, MODEL_TSMFK01, MODEL_B14
+from lib.dispersion.variables import MODEL_CR72, MODEL_IT99, MODEL_TSMFK01, MODEL_B14
 # Analytical full
-from specific_analyses.relax_disp.variables import MODEL_CR72_FULL, MODEL_B14_FULL
+from lib.dispersion.variables import MODEL_CR72_FULL, MODEL_B14_FULL
 # NS : Numerical Solution
-from specific_analyses.relax_disp.variables import MODEL_NS_CPMG_2SITE_3D, MODEL_NS_CPMG_2SITE_STAR, MODEL_NS_CPMG_2SITE_EXPANDED
+from lib.dispersion.variables import MODEL_NS_CPMG_2SITE_3D, MODEL_NS_CPMG_2SITE_STAR, MODEL_NS_CPMG_2SITE_EXPANDED
 # NS full
-from specific_analyses.relax_disp.variables import MODEL_NS_CPMG_2SITE_3D_FULL, MODEL_NS_CPMG_2SITE_STAR_FULL
+from lib.dispersion.variables import MODEL_NS_CPMG_2SITE_3D_FULL, MODEL_NS_CPMG_2SITE_STAR_FULL
 
 # Analysis variables.
 ##################################################################################
@@ -122,9 +119,9 @@ if not hasattr(ds, 'tmpdir'):
 if not hasattr(ds, 'resdir'):
     ds.resdir = None
 
-# Do set_grid_r20_from_min_r2eff.
-if not hasattr(ds, 'set_grid_r20_from_min_r2eff'):
-    ds.set_grid_r20_from_min_r2eff = True
+# Do r20_from_min_r2eff.
+if not hasattr(ds, 'r20_from_min_r2eff'):
+    ds.r20_from_min_r2eff = True
 
 # Remove insignificant level.
 if not hasattr(ds, 'insignificance'):
@@ -423,7 +420,7 @@ for exp_type, frq, ei, mi in loop_exp_frq(return_indices=True):
 
         ### Now back calculate values from parameters, and stuff R2eff it back.
         print("Generating data with MODEL:%s, for spin id:%s"%(model_create, cur_spin_id))
-        r2effs = optimisation.back_calc_r2eff(spin=cur_spin, spin_id=cur_spin_id)
+        r2effs = optimisation.back_calc_r2eff(spins=[cur_spin], spin_ids=[cur_spin_id])
 
         # Define file name
         file_name = "%s%s.txt" % (exp_id, cur_spin_id .replace('#', '_').replace(':', '_').replace('@', '_'))
@@ -470,24 +467,24 @@ relax_disp.insignificance(level=ds.insignificance)
 if ds.GRID_INC:
     # Set the R20 parameters in the default grid search using the minimum R2eff value.
     # This speeds it up considerably.
-    if ds.set_grid_r20_from_min_r2eff:
-        relax_disp.set_grid_r20_from_min_r2eff(force=False)
+    if ds.r20_from_min_r2eff:
+        relax_disp.r20_from_min_r2eff(force=False)
 
     # Then do grid search.
-    grid_search(lower=None, upper=None, inc=ds.GRID_INC, constraints=True, verbosity=ds.verbosity)
+    minimise.grid_search(lower=None, upper=None, inc=ds.GRID_INC, constraints=True, verbosity=ds.verbosity)
 
 # If no Grid search, set the default values.
 else:
     for param in MODEL_PARAMS[model_analyse]:
         value.set(param=param, index=None)
         # Do a grid search, which will store the chi2 value.
-    #grid_search(lower=None, upper=None, inc=10, constraints=True, verbosity=ds.verbosity)
+    #minimise.grid_search(lower=None, upper=None, inc=10, constraints=True, verbosity=ds.verbosity)
 
 # Save result.
 ds.grid_results = save_res(cur_spins)
 
 ## Now do minimisation.
-minimise(min_algor='simplex', func_tol=ds.set_func_tol, max_iter=ds.set_max_iter, constraints=True, scaling=True, verbosity=ds.verbosity)
+minimise.execute(min_algor='simplex', func_tol=ds.set_func_tol, max_iter=ds.set_max_iter, constraints=True, scaling=True, verbosity=ds.verbosity)
 
 # Save results
 ds.min_results = save_res(cur_spins)
@@ -512,7 +509,7 @@ if ds.do_cluster:
     relax_disp.parameter_copy(pipe_from=ds.pipe_name_MODEL, pipe_to=ds.pipe_name_MODEL_CLUSTER)
 
     # Now minimise.
-    minimise(min_algor='simplex', func_tol=ds.set_func_tol, max_iter=ds.set_max_iter, constraints=True, scaling=True, verbosity=ds.verbosity)
+    minimise.execute(min_algor='simplex', func_tol=ds.set_func_tol, max_iter=ds.set_max_iter, constraints=True, scaling=True, verbosity=ds.verbosity)
 
     # Save results
     ds.clust_results = save_res(cur_spins)
