@@ -22,21 +22,19 @@
 # Module docstring.
 """Module for relax version information."""
 
-# Dependencies.
-import dep_check
-
 # Python module imports.
 from os import F_OK, access, sep
-from re import search
-PIPE, Popen = None, None
-if dep_check.subprocess_module:
+try:
     from subprocess import PIPE, Popen
+except ImportError:
+    PIPE, Popen = None, None
 
 # relax module imports.
+import lib.structure.internal.object
 from status import Status; status = Status()
 
 
-version = "3.2.1"
+version = "3.3.1"
 repo_revision = None
 repo_url = None
 
@@ -116,3 +114,6 @@ def version_full():
 
 # Fetch the repository information, if present.
 repo_information()
+
+# Set the version in the relax internal structural object.
+lib.structure.internal.object.RELAX_VERSION = version_full()
